from .en import *



class TitlePageComponent(Environment):
    
    latex_name='titlepage'
    
    def __init__(self, system=None, options=None, arguments=None, start_arguments=None,
                 **kwargs):
        r"""
        Args
        ----
        options: str or list or  `~.Options`
            Options to be added to the ``\begin`` command
        arguments: str or list or `~.Arguments`
            Arguments to be added to the ``\begin`` command
        start_arguments: str or list or `~.Arguments`
            Arguments to be added before the options
        """

        self.system = system
        self.options = options
        self.arguments = arguments
        self.start_arguments = start_arguments

        
        
        super().__init__(options=options, arguments=arguments, start_arguments=start_arguments,**kwargs)
        
        if self.system is not None:

        
            system = self.system


            
            self.append(NoEscape('\centering'))

            self.append(NoEscape('\\Huge DRGANIA MECHANICZNE \n \n'))
            
            self.append(Command('vspace',arguments='1cm'))

            
            
            if len(system.q)==1:
                dof_str = 'JEDNYM STOPNIU SWOBODY'
            else:
                dof_str = 'WIELU STOPNIACH SWOBODY'
                
            if system._dissipative_potential==0 or system._dissipative_potential is None:
                damping_str = 'NIETŁUMIONE'
            else:
                damping_str = 'TŁUMIONE'

           
            self.append(NoEscape(f'\\Large {damping_str} UKŁADY O {dof_str} \n \n'))    
            
            self.append(Command('vspace',arguments='1cm'))
            
            self.append(NoEscape(f'{system._label} \n \n'))
            
            self.append(Command('vspace',arguments='1cm'))
            
            self.append(Command('MyAuthor'))
            self.append(NoEscape(f'\\par'))
            self.append(Command('vspace',arguments='1cm'))
            
            #self.append(Command('vspace',arguments='1cm'))
            #self.append(NoEscape(f'\\protect\\par'))
            #self.append(NewLine())
            self.append(Command('MyDate'))


            
######################### Initial point
    
    
# Damian
class ExemplaryPictureComponent(ReportComponent):
    
    title="Przykład rzeczywistego obiektu"
    packages=[Package('float')] 
    @property
    def header_text(self):
        
        return "Ilustracja przedstawia schemat rzeczywistego obiektu mechanicznego, wyznaczony na podstawie uprzedniej analizy rzeczywistego obiektu."
    @property
    def footer_text(self):


        return "Model dynamiczny układu określa się na podstawie analizy rozważanego przypadku. Należy pamiętać, że stopień odwzorowania (poziom abstrakcji) modelu zależy od tego do czego planuje się go używać."
    
    def append_elements(self):
        
        system = self._system

        display(ReportText(self.header_text))

        with self.create(Figure(position='H')) as fig:
            fig.add_image(system._real_example(),width='8cm')

        display(ReportText(self.footer_text))

# Boogi        
class SchemeComponent(ExemplaryPictureComponent):
    title="Schemat systemu"

    @property
    def header_text(self):
        #
        return "Ilustracja przedstawia schemat rzeczywistego obiektu mechanicznego, wyznaczony na podstawie uprzedniej analizy rzeczywistego obiektu."

        
    @property
    def footer_text(self):
        
        system = self._system

        return f"Analizując przedstawiony układ można stwierdzić, że jego liczba stopni swobody to {len(system.q)}."
    
    
    def append_elements(self):
        
        system = self._system

        display(ReportText(  self.header_text ))
          
        with self.create(Figure(position='H')) as fig:
            fig.add_image(system._scheme(),width='8cm')

        display(ReportText( self.footer_text ))



class NumericalAnalysisComponent(ExemplaryPictureComponent):
    title="Symulacja numeryczna"

    _default_ics = None
    _default_parameters = {}
    _default_tspan = np.linspace(0,1,1000)
    
    @classmethod
    def set_default_ics(cls,default_ics=None):
        
        if default_ics is None:
            cls._default_ics = default_ics
        
        return cls

    @classmethod
    def set_default_parameters(cls,default_parameters=None):
        
        if default_parameters is None:
            cls._default_parameters = default_parameters
        
        return cls
    
    @classmethod
    def set_default_tspan(cls,default_tspan=None):
        
        if default_tspan is None:
            cls._default_tspan = default_tspan
        
        return cls
    

    def append_elements(self):
        
        system = self._system

        display(ReportText(f'''Dla Damiana :P
                            '''))
        if self._default_ics is None:
            ics = list(system.Y*0.0)
        else:
            ics = self._default_ics
            
        sym_res = system.subs(self._default_parameters).numerized().compute_solution(self._default_tspan,ics)

        LatexDataFrame.formatted(sym_res).plotted(preview=True)
        

        display(ReportText(f'''Dla Damiana :P
                            '''))

# Boogi
class KineticEnergyComponent(ReportComponent):
    
    title="Energia kinetyczna"

    @property
    def header_text(self):
        #"Energia kinetyczna układu wyrażona jest wzorem:"
        return "Energia kinetyczna układu wyrażona jest wzorem:"

        
    @property
    def footer_text(self):
        #"Wyznaczona wielkość określa energię układu wynikającą z jego własności inercyjnych (energię zmagazynowaną w elementach bezwładnych)."
        return "Determined formula specify energy of the system related to its inertial properties."
    
    def append_elements(self):
        
        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys




        display(ReportText( self.header_text  ))
        

        display(SympyFormula( Eq(Symbol('T'),
                     dyn_sys_lin._kinetic_energy) , marker=None))
               
        display(ReportText( self.footer_text  ))
    
    
    
class PotentialEnergyComponent(ReportComponent):#Jaś fasola
    
    title="Energia potencjalna"
    @property
    def header_text(self):
        
        return "Energia potencjalna układu wyrażona jest wzorem:"

        
    @property
    def footer_text(self):
        
        return "Zaprezentowana zależność opisuje oddziaływanie potencjalnych pól sił w których znajduje się obiekt."

    def append_elements(self):

        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(  self.header_text   ))

        display(SympyFormula( Eq(Symbol('V'),
                     dyn_sys_lin._potential_energy ), marker=None))

        display(ReportText(  self.footer_text   ))

# Marcel spolszczone
class DissipationComponent(ReportComponent):
    
    
    title="Dyssypacyjna funkcja Rayleigh'a"
    
    
    @property
    def header_text(self):
        
        return "Energia rozpraszana tłumieniem wyrażona jest wzorem:"

        
    @property
    def footer_text(self):
        
        return "Podana zależność stanowi potencjał dysynpacyjny Rayleigh'a, który poddany różniczkowaniu względem wektora prędkości uogólnionych pozwala na określenie sił wiskotycznego tłumienia."
    
    def append_elements(self):
        
        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText( self.header_text  ))
        

        display(SympyFormula( Eq(Symbol('D'),
             dyn_sys_lin._dissipative_potential) , marker=None ))

        display(ReportText( self.footer_text  ))

# Boogi          
class LagrangianComponent(ReportComponent):
    
    #title="Lagrangian (funkcja Lagrange'a)  układu"
    title="Lagrangian of the system (Lagranges function)"
        
    @property
    def header_text(self):
        #"Lagrangian systemu wyrażony jest wzorem ({AutoMarker(Eq(Symbol('L'),dyn_sys.L.expand()[0]))}):"
        
        system = self._system
        dyn_sys=system
        
        return f"System Lagrangian is described by the formula ({AutoMarker(Eq(Symbol('L'),dyn_sys.L.expand()[0]))}):"
        
    @property
    def body_text_1(self):
        #"Równania Eulera-Lagrange'a dla rozważanego przypadku są nastęujące: "   
        return "The Euler-Lagrange equations for the case under consideration are as follows: "
        
    @property
    def body_text_2(self):
        #"Kolejne pochodne wynikające z zastosowania równań Eulera-Lagrange'a są nastęujące: "
        return "Subsequent derivatives obtained with the Euler-Lagrange equations are as follows: "
    
    @property
    def footer_text(self):
        #"Wyniki przedstawionych operacji wykorzystuje się wyznaczenia równań ruchu układu."
        return "The results of the presented operations are used to determine the equations of motion of the system."
        
    def append_elements(self):
        
        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys.linearized()

        
        print(system._scheme())
        
        mrk_lagrangian_nonlin=Marker('lagrangianNL',prefix='eq')

        #display(ReportText(f'''The following model is considered. The system's Lagrangian is described by the formula ({Ref(mrk_lagrangian_nonlin).dumps()}):
        #                    '''))
        display(ReportText(self.header_text ))

        display((SympyFormula(  Eq(Symbol('L'),dyn_sys.L.expand()[0])  , marker=mrk_lagrangian_nonlin )  ))
        
        q_sym =[ Symbol(f'{coord}'[0:-3]) for coord in dyn_sys.q]
        
        diffL_d=lambda coord: Symbol(f'\\frac{{ \\partial L}}{{  \\partial {vlatex(coord)}  }}')
        diffD_d=lambda coord: Symbol(f'\\frac{{ \\partial D}}{{  \\partial {vlatex(diff(coord))}  }}')
        d_dt_diffL_d=lambda coord: Symbol(f'\\frac{{ \\mathrm{{d}}  }}{{  \\mathrm{{d}} {vlatex(system.ivar)}  }} {vlatex(diffL_d(coord))} ')        

        display(ReportText(self.body_text_1))
        
        for coord in dyn_sys.q:
            display((SympyFormula(  Eq(d_dt_diffL_d(coord.diff(system.ivar)) - diffL_d(coord) + diffD_d(coord),Symbol(f'Q_{{ {vlatex(coord)} }}^N'))  , marker=mrk_lagrangian_nonlin,backend=vlatex )  ))
        
        
        display(ReportText(self.body_text_2))
        
        for coord in dyn_sys.Y:
            display((SympyFormula(  Eq(diffL_d(coord),dyn_sys.L.expand()[0].diff(coord))  , marker=mrk_lagrangian_nonlin,backend=vlatex )  ))
            


        for coord in dyn_sys.q.diff(system.ivar):
            display((SympyFormula(  Eq(d_dt_diffL_d(coord),dyn_sys.L.expand()[0].diff(coord).diff(system.ivar))  , marker=mrk_lagrangian_nonlin,backend=vlatex )  ))
        for coord in dyn_sys.q:
            display((SympyFormula(  Eq(diffD_d(coord),(S.One/2 * diff(dyn_sys.q.transpose())* dyn_sys_lin.damping_matrix()* diff(dyn_sys.q))[0].diff(diff(coord)))  , marker=mrk_lagrangian_nonlin,backend=vlatex )  ))
            #display(Markdown(f'\\begin{equation}    \\end{equation}').reported())
        #with doc_model.create(DMath()) as eq:
        #    eq.append(NoEscape(latex(Derivative(Symbol('L'),q_sym[0],evaluate=False))))
        #    eq.append(NoEscape('='))
        #    eq.append(NoEscape(vlatex(dyn_sys.L.expand()[0].diff(dyn_sys.q[0]))))

        mrk_gov_eq_nonlin=Marker('gov_eq_nonlin_sys',prefix='eq')

        #display(ReportText(f'''The governing equations of the system have a following form ({Ref(mrk_gov_eq_nonlin).dumps()}):
        #                    '''))

        display(ReportText(self.footer_text))


        
    def as_frame(self):
        frame=Frame(title=self.title,options=['allowframebreaks'])
        #frame.packages +(self.packages)
        frame+=(list(self))
        return frame

        
    def as_frame(self):
        frame=Frame(title=self.title,options=['allowframebreaks'])
        #frame.packages +(self.packages)
        frame+=(list(self))
        return frame
    
# Amadi & Damian
class GoverningEquationComponent(ReportComponent):
    
    #Równania ruchu
    title="Equation of motion"
    
    @property
    def entry_text_one(self):
        system = self._system
        dyn_sys=system
        return f"Wykorzystując obliczone pochodne, wyznacza się równanie ruchu na podstawie odpowiedniego wzoru. Równanie ruchu układu przedstawia zależność: ({AutoMarker(Eq(dyn_sys._eoms[0].simplify().expand(),0))})"
    
    @property
    def entry_text_two(self):
        system = self._system
        dyn_sys=system
        
        return f"Wykorzystując obliczone pochodne, wyznacza się równania ruchu na podstawie odpowiedniego wzoru. Równania ruchu układu przedstawiają zależności: ({AutoMarker(Eq(dyn_sys._eoms[0].simplify().expand(),0))})-     ({AutoMarker(Eq(dyn_sys._eoms[-1].simplify().expand(),0))})"

    @property
    def footer_text(self):
        #f''' Wyznaczone równania stanowią matematyczny opis dynamiczny właściwości układu. Dalsza analiza pozwala na skuteczną analizę działania modelowanego obiektu i określenie jego parametrów mechanicznych. '''
        return f'''  Wyznaczone równania stanowią matematyczny opis dynamiczny właściwości układu. Dalsza analiza pozwala na skuteczną analizę działania modelowanego obiektu i określenie jego parametrów mechanicznych. '''
    
    def append_elements(self):
        
        system = self._system
        dyn_sys=system

        if len(system.q)==1:
            display(ReportText(self.entry_text_one))   
        else:
            display(ReportText(self.entry_text_two))   

        
        for eq in dyn_sys._eoms:
            display(SympyFormula( Eq(eq.simplify().expand(),0) , marker=None))

        display(ReportText(self.footer_text))   

class LinearizedGoverningEquationComponent(ReportComponent):
    #Równania ruchu
    title="Equation of motion"

    @property
    def entry_text_one(self):
        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys.linearized()
        
        return f"Wykorzystując obliczone pochodne, wyznacza się równanie ruchu na podstawie odpowiedniego wzoru. Zlinearyzowane równanie ruchu układu przedstawia zależność: ({AutoMarker(Eq(dyn_sys_lin._eoms[0].simplify().expand(),0))})"
    
    @property
    def entry_text_two(self):
        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys.linearized()
        
        return f"Wykorzystując obliczone pochodne, wyznacza się równania ruchu na podstawie odpowiedniego wzoru. Zlinearyzowane równania ruchu układu przedstawiają zależności: ({AutoMarker(Eq(dyn_sys_lin._eoms[0].simplify().expand(),0))})-({AutoMarker(Eq(dyn_sys_lin._eoms[-1].simplify().expand(),0))})"
    
    @property
    def footer_text(self):
        
        return f''' Wyznaczone równania stanowią matematyczny opis dynamiczny właściwości układu. Dalsza analiza pozwala na skuteczną analizę działania modelowanego obiektu i określenie jego parametrów mechanicznych. '''
    
    def append_elements(self):
        
        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys.linearized()


        
        if len(system.q)==1:
            display(ReportText(self.entry_text_one))   
        else:
            display(ReportText(self.entry_text_two))   
        
        for eq in dyn_sys._eoms:
            display(SympyFormula( Eq(eq.simplify().expand(),0) , marker=None))

        display(ReportText(self.footer_text))   
        

class LinearizationComponent(ReportComponent): # Szymon

    title="Linearyzacja równań ruchu"

    @property
    def entry_text(self):
        return "Linearyzaja równań polega na znalezieniu ich rozwinięcia w szereg Taylora względem współrzędnych, prędkości i przyspieszeń uogólnionych w otoczeniu punktu równowagi. Celem uproszczenia wprowadzono następujące oznaczenia:"

    @property
    def equilibrium_point_text(self):
        return "Punkty równowagi rozważanego układu są następujące:"
    @property
    def eom_text(self):
        dyn_sys= self._system
        return '''Równanie ruchu dla współrzędnej ${latex(dyn_sys.q[no])}$ można przestawić jako:'''
    @property
    def lagrange_text(self):
        dyn_sys= self._system
        return "Formalnie należy obliczyć pochodne cząstkowe wielkości uogólnionych ze składników równań Lagrange'a"

    @property
    def derivative_text(self):
        return "Poszczególne pochodne mają następującą postać:"

    @property
    def linearized_eq_text(self):
        return "Po podstawieniu obliczonych pochodnych, otrzumuje się następujące zlinearyzowane równanie:"

    def append_elements(self):

        system = self._system
        ReportText.set_directory('./SDAresults')
        latex_store=AutoBreak.latex_backend
        AutoBreak.latex_backend = latex_store
        mrk_lagrangian_nonlin = Marker('lagrangLin',prefix='eq')
        mrk_lagrangian_lin = Marker('lagrangLin',prefix='eq')

        t=system.ivar

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()
        coords=tuple(list(dyn_sys.Y) + list(dyn_sys.q.diff(t,t)))
        op_point = {coord: 0 for coord in coords}
        op_point.update(dyn_sys._op_points(subs=True)[0])

        display(ReportText(self.entry_text))


        for coord in coords:
            display((SympyFormula(  Eq(Symbol(vlatex(coord)),Symbol(latex(coord))) , marker=None  )  )) 



        display(ReportText( self.equilibrium_point_text ))

        for eq_coord,val in op_point.items():
            display((SympyFormula(  Eq(eq_coord,val) , marker=mrk_lagrangian_lin  )  ))


        diffL_d=lambda coord: Symbol(latex(Derivative(Symbol('L'),Symbol(vlatex(coord))))  )



        for no,eom in enumerate(dyn_sys._eoms):




            eq_sym=Symbol(f'RR_{latex(dyn_sys.q[no])}')


            display(ReportText( self.eom_text.format(coord = latex(dyn_sys.q[no])) ))

            #display((SympyFormula(  Eq(eq_sym,eom,evaluate=False) , marker=mrk_lagrangian_lin,backend=latex  )  ))


            display(ReportText( self.lagrange_text ))


            display((SympyFormula(  Eq(comps.MultivariableTaylorSeries(eq_sym,coords,n=1,x0=op_point)._symbolic_sum(),0) , marker=None,backend=latex  )  ))

            diff_list=comps.MultivariableTaylorSeries(eom,coords,n=1,x0=op_point).calculation_steps(expr_symbol=eq_sym)

            display(ReportText( self.derivative_text ))

            for diff_eq in diff_list:

                display((SympyFormula(  diff_eq , marker=mrk_lagrangian_lin,backend=latex  )  ))

            display(ReportText( self.linearized_eq_text ))
            display((SympyFormula(  Eq(comps.MultivariableTaylorSeries(eom,coords,n=1,x0=op_point).doit().expand().simplify().expand(),0,evaluate=False) , marker=mrk_lagrangian_lin,backend=latex  )  ))



        AutoBreak.latex_backend = latex_store


# Marcel & Monika spolszczone
class FundamentalMatrixComponent(ReportComponent):
    
    title="Wyznaczanie macierzy fundamentalnej";

    
    @property
    def header_text(self):
        return "Z równań ruchu wyznaczono macierz mas i sztywności układu::"

        
    @property
    def body_text(self):

        return "Macierz fundamentalna, na podstawie której wyznaczono równanie charakterystyczne rozważanego układu ${latex(Delta)}$, przedstawiają się następująco::"
    
    @property
    def footer_text(self):

        return "Macierz fundamentalna pozwala określić rozwiązanie ustalone. Natomiast bazując na równaniu charakterystycznym określa się częstości własne układu."
    
    
    def append_elements(self):
        
        system = self._system
        ReportText.set_directory('./SDAresults')

        latex_store=AutoBreak.latex_backend
        AutoBreak.latex_backend = latex_store
        
        t=system.ivar
        

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()


        display(ReportText(self.header_text))
        
        display((SympyFormula(  Eq(Symbol('M'),dyn_sys_lin.inertia_matrix(),evaluate=False) , marker='a' )  ))

        display((SympyFormula(  Eq(Symbol('K'),dyn_sys_lin.stiffness_matrix(),evaluate=False) , marker='a')  ))

        Delta = Symbol('\Delta')

        display(ReportText(self.body_text))

        display((SympyFormula(  Eq(Symbol('A'),dyn_sys_lin.fundamental_matrix(),evaluate=False) , marker='a'  )  ))
        display((SympyFormula(  Eq(Delta,dyn_sys_lin.fundamental_matrix().det().expand().simplify().simplify().expand(),evaluate=False) , marker='a',backend=latex  )  ))

        display(ReportText(self.footer_text))

        AutoBreak.latex_backend = latex_store

# Mateusz
class GeneralSolutionComponent(ReportComponent):
    title="Rozwiązanie ogólne"
    
    @property
    def header_text(self):
        
        return 'Rozwiązanie ogólne przedstawia wyrażenie:'
    @property
    def footer_text(self):
        
        return 'Rozwiązanie ogólne opisuje ruch analizowanego układu (przedstawia przemieszczenie w funkcji czasu) i wynika z rozważań dotyczących drgań swobodnych układu.'
    
    def append_elements(self):

        from ....dynamics import LagrangesDynamicSystem, HarmonicOscillator
        #from dynpy import LagrangesDynamicSystem, HarmonicOscillator

        system = self._system
        ReportText.set_directory('./SDAresults')

        latex_store=AutoBreak.latex_backend
        
        
        AutoBreak.latex_backend = latex_store
        
        t=system.ivar
        

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()


        display(ReportText(self.header_text))
        display((SympyFormula(  Eq(Symbol('X'),HarmonicOscillator(dyn_sys_lin.linearized(
                                            )).general_solution().n(3),
                                            evaluate=False) , marker='a',backend=latex  )  ))

        display(ReportText(self.footer_text))

        AutoBreak.latex_backend = latex_store

        
# Grześ
class FrequencyResponseFunctionComponent(ReportComponent):
    #"amplitude-frequency characteristic"
    title="Charakterystyka Amplitudowo-Częstotliwościowa"
    
    @property
    def header_text(self):
        #"funkcja odpowiedzi częstotliwościowej:"
        return "funkcja odpowiedzi częstotliwościowej:"
    @property
    def footer_text(self):
        #"jest to suma kwadratów amplitud pod pierwiastkiem:"
        return "jest to suma kwadratów amplitud pod pierwiastkiem:"

    def append_elements(self):

        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('frf'),
                     dyn_sys.frequency_response_function() ), marker=None))

        display(ReportText(self.footer_text))
        
FRFComponent = FrequencyResponseFunctionComponent



class FrequencyResponseFunctionComponentToSecond(ReportComponent):
    
    title="sens mocy układu"

    def append_elements(self):

        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(f'''
                           sens mocy układu:
                           '''))

        display(SympyFormula( Eq(Symbol('frf^2'),
                     dyn_sys.frequency_response_function().doit()**2 ), marker=None))

        display(ReportText(f'''
                           Charakterystyka Amplitudowo-Częstotliwościowa podniesiona do kwadratu
                           '''))  
        


        
# Marcel & Monika
class SteadySolutionComponent(ReportComponent):
    
        #"Rozwiązanie szczególne"
    title="Steady solution"
    _phi=False
    
    @property
    def header_text(self):
        #"Rozwiązanie szczególne przedstawia wyrażenie:"
        # google tlumacz
        return "The steady solution is given by the formula:"

        
    @property
    def footer_text(self):
        #" Rozwiązanie szczególne związane jest obecnością wielkości wymuszających ruch (drgania) analizowanego układu."
        # google tlumacz
        return "The specific solution is related to the presence of quantities that force motion (vibrations) of the analyzed system."
    
    def append_elements(self,phi=_phi):

        from ....dynamics import LagrangesDynamicSystem, HarmonicOscillator
        
        system = self._system
        ReportText.set_directory('./SDAresults')

        latex_store=AutoBreak.latex_backend
        AutoBreak.latex_backend = latex_store
        
        t=system.ivar
        

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()



        display(ReportText(self.header_text ))

        display((SympyFormula(  Eq(Symbol('X_s'),
                            HarmonicOscillator(dyn_sys_lin.linearized(
                            )).steady_solution().n(3),
                            evaluate=False) , marker='b',backend=latex  )  ))

        AutoBreak.latex_backend = latex_store

        display(ReportText(self.footer_text ))
        
#### Amadi
class MaxStaticForce(ReportComponent):
    
    #Maksymalna siła statyczna
    title="Maximum static force"

    @property
    def header_text(self):
        
        return "Wartość maksymalna siły statycznej działającej na pojedynczy element mocujący:"
    
    
    def append_elements(self):

        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F_s'),
                     dyn_sys.max_static_force().doit() ), marker=None))
        
#### Amadi
class MaxDynamicForce(ReportComponent):
    
    #Maksymalna siła statyczna
    title="Maximum dynamic force"

    @property
    def header_text(self):
       
        return "Wartość maksymalna siły dynamicznej działającej na pojedynczy element mocujący:"
    
    def append_elements(self):

        system = self._system
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F_d'),
                     dyn_sys.max_dynamic_force().doit() ), marker=None))

##### Karolina + Kamil + Bogi rr
class SeparableODEIntroComponent(ReportComponent):
    title="Podstawowa wiedza niezbędna do rozwiązania równania o zmiennych rozdzielonych"
    def append_elements(self):
        system = self.reported_object
        ode_sys =system

        display(ReportText('Jednym z fundamentalnych typów równań różniczkowych jest równanie o zmiennych rozdzielonych. Proces rozwiązania tego typu równania polega na separacji wyrazów zawierających $y$ i $x$, przeniesieniu ich na różne strony równania, a następnie zastosowaniu operacji całkowania. Po przeprowadzeniu całkowania nie można zapomnieć o dodaniu do jednej ze stron stałej całkowania. Poniżej znajduje się przykład zadania ilustrującego proces rozwiązania tego rodzaju równania.:'))

class EquationDefinitionComponent(ReportComponent):
    title="Rozpatrujemy następujące równanie różniczkowe: "

    def append_elements(self):

        system = self.reported_object
        ode_sys =system

        y_sym = system.dvars[0]
        display(SympyFormula(Eq(ode_sys.lhs[0],ode_sys.rhs[0]).subs(ode_sys._get_dvars_symbols())))
        display(ReportText('Gdzie:'))
        display(SympyFormula(y_sym))
        display(ReportText('Jest naszą szukaną funkcją.'))

class VariablesSeparationComponent(ReportComponent):
    
    #title="Separation of variables"
    title="Rodzielanie zmiennych"

    def append_elements(self):

        system = self.reported_object._hom_equation()
        ode_sys =system
        ############################ must have

#         display(ReportText(  self.header_text   ))

        fun = system.dvars[0]
        ivar = system.ivar

        fun_str = system._get_dvars_symbols()[fun.diff(ivar)]

        d_ivar = system._variable_name()[0]
        d_var = system._variable_name()[1]

        y_sym = system._get_dvars_symbols()[fun]
        gx=Symbol('g(x)')
        hx=Symbol('h(x)')

        
        
        #ode_rhs = solve(ode_sys.lhs[0],ode_sys.dvars[0].diff())[0]
        
        #sep_vars_dict = (separatevars(ode_rhs.subs(fun,y_sym),(ivar,y_sym),dict=True))
        
        g_fun_ivar = system._function_separation()[0]
        h_fun_dvar = system._function_separation()[1]
        #display(ReportText('If it is possible variables $x$ and $y$ should be separated like shown below:'))
        display(ReportText('Jeżeli jest to możliwe, dokonujemy powszechnie znanego rozdzielenia zmiennych $x$ i $y$. Poprzez rozdzielenie zmiennych rozumie się oddzielenie wyrazów zawierających zmienną $x$ od tych, które zawierają zmienną $y$. W celu dokonania tego podziału, możemy przyporządkować te wyrazy do funkcji $h(x)$ i $g(x)$, co pokazano poniżej:'))
        display(SympyFormula(Eq(gx,g_fun_ivar)))
        display(SympyFormula(Eq(hx,h_fun_dvar)))
        display(ReportText(f'Poza tymi elementami nasze równanie zawiera również wyraz ${latex(fun_str)}$, który można również zapisać jako:'))
        display(SympyFormula(Eq(fun_str,d_var/d_ivar)))
        display(ReportText(f'Po wstawieniu tego wyrażenia można zauważyć, że w pózniejszych krokach pomnożenie równania obustronnie przez ${latex(d_ivar)}$ umożliwia scałkowanie obu stron względem innych zmiennych.'))

class SeparatedVariablesIntegrationComponent(ReportComponent):
    
    #title="Integration of separated variables"
    title="Całkowanie zmiennych rozdzielonych"
    def append_elements(self):

        system = self.reported_object._hom_equation()
        ode_sys =system
        ############################ must have

#         display(ReportText(  self.header_text   ))

        fun = system.dvars[0]
        ivar = system.ivar
        gx=Symbol('g(x)')
        hx=Symbol('h(x)')
        

        d_ivar = system._variable_name()[0]
        d_var = system._variable_name()[1]
        y_sym = system._variable_name()[2]
        fun_str = system._variable_name()[3]

        


        

        
        g_fun_ivar = system._function_separation()[0]
        h_fun_dvar = system._function_separation()[1]
        #display(ReportText(f'We treat our ${latex(hx)}$ function as a ${latex(y_sym)}$'))
        display(ReportText('Poniższe równanie należy zmodyfikować:'))###nie jestem pewien co tutaj wsyawic jako ten opis
        
        display(SympyFormula( Eq(Symbol(latex(d_var/d_ivar)),g_fun_ivar*h_fun_dvar)))

        #display(ReportText('Two functions of seprarate variables are to define. These are as follows'  ))
        display(ReportText(f'W tym celu mnożymy obustronnie przez ${latex(d_ivar)}$ oraz przenosimy ${latex(fun)}$ na przeciwną stronę równania:'  ))
        
        display(SympyFormula( Symbol(latex(d_var/h_fun_dvar)+ '=' + latex(g_fun_ivar) + latex(d_ivar) )   ) )
        #display(ReportText('Integration on both sides'))## uzupełnić opis
        display(ReportText('''Następnie, aby "wrócić" do funkcji pierwotnej oraz otrzymać wynik w postaci funkcji $y$ całkujemy otrzymane równanie obustronnie: '''))
        
        #display(SympyFormula(  Symbol('\\int ' + latex(d_var/y_sym) + '=  \\int ' + latex(g_fun_ivar) + latex(d_ivar) )   ) )
        display(ReportText('''Co ważne, przy całkowaniu należy pamiętać o dodaniu stałej całkowania w dowolnej postaci naogół zapisywanej jako $C$ bądź $D$. '''))
        display(SympyFormula(  Symbol(latex( Integral(1/h_fun_dvar,y_sym) )+ '= ' + latex(Integral(g_fun_ivar,ivar))  )   ) )

#         const = (system.solution._spot_constant())[0]
        const=Symbol('D')
        display(ReportText(' Po scałkowaniu otrzymujemy następujący wynik:'))
        display(SympyFormula(  Symbol(latex( Integral(1/h_fun_dvar,y_sym).doit() )+ '=' + latex(Integral(g_fun_ivar,ivar).doit()) + '+' + latex(const)   )   ) )
#         display(ReportText('Equations presented above correspond to each generalized coordinate:'))
#         display(SympyFormula(system.dvars))

#         display(ReportText(  self.footer_text   ))

class SolutionComponent(ReportComponent):
    
    title="Uproszczenie rozwiązania otrzymanego po scałkowaniu równań"

    def append_elements(self):

        system = self.reported_object
        ode_sys =system
        ############################ must have

#         display(ReportText(  self.header_text   ))

        fun = system.dvars[0]
        ivar = system.ivar

        fun_str = system._variable_name()[3]

        d_ivar = system._variable_name()[0]
        d_var = system._variable_name()[1]

        y_sym = system._variable_name()[2]


        
        ode_rhs = solve(ode_sys.lhs[0],ode_sys.dvars[0].diff())[0]
        
        sep_vars_dict = (separatevars(ode_rhs.subs(fun,y_sym),(ivar,y_sym),dict=True))
        
        g_fun_ivar = system._function_separation()[0]
        h_fun_dvar = system._function_separation()[1]
        
        #display(ReportText('After rearrangment a solution has the following form:'  ))
        display(ReportText('Po uporządkowaniu oraz przeobrażeniu stałej wyrażenie ma następującą postać:'  ))## trzeba rozwinąć hasło przeobrażenie
        display(SympyFormula( system.general_solution.as_eq_list()[0] ))

        #display(ReportText('Obtained expresions enable to rewrite equation under consideration.'  ))
        display(ReportText('Otrzymane wyrażenie stanowi rozwiązania równania różniczkowego o zmiennych rozdzielonych. Stała całkowania jest możliwa do policzenia w momencie w którym w zadaniu zostały podane warunki początkowe, czyli wartość funkcji dla konkretnego $x$.'  ))

class ClassifyODEComponent(ReportComponent):
    title="Klasyfikacja równania różniczkowego"

    def append_elements(self):

        system = self.reported_object
        types=system._classify_ode()
        display(ReportText("To równanie różniczkowe możemy określić jako równanie typu:"))
        for feature in types:
            display(ReportText(f'\n\n - { feature } '.replace('_',' ')))
        

class LinearODEIntroComponent(ReportComponent):
    title="Podstawowa wiedza niezbędna do rozwiązania równania różniczkowego"
    
    def append_elements(self):

        system = self.reported_object
        ode_sys =system
        fun = system.dvars[0]
        ivar = system.ivar

        fun_str = system._variable_name()[3]

        d_ivar = system._variable_name()[0]
        d_var = system._variable_name()[1]

        y_sym = system._variable_name()[2]
        Px=Symbol('P(x)')
        Qx=Symbol('Q(x)')
        n=Symbol('n')
        display(ReportText('Jednym z najbardziej podstawowych typów równań różniczkowych jest równanie liniowe o stałych współczynnikach. Równanie liniowe składa się jedynie z liniowych wyrazów, które mogą zawierać stałe współczynniki. Wymagane jest przejście na rówanie jednorodne- przenosimy wszystkie wyrazy z funkcją $y$ i jej pochodnymi, po czym przyrównujemy do zera otrzymując równanie jednorodne. Następnie wymagane jest uzmiennienie stałej i traktowanie jej jako funkcji zależnej od $x$. Liczymy pochodne obu stron i wykorzystujemy wyniki w początkowym równaniu. Wynikiem jest suma rozwiązań z obu części')) ###dodać: mające postać a*y'+b*y+R(x),gdzie r(x) to funckja x opisana jawnie gdzie a=... b=... r(x)=... i tutaj wpisać dla naszego przykładu z tego spot consant)

class LinearTransformation(ReportComponent):
    title=" Przejscie na równanie jednorodne"
    
    def append_elements(self):
        system = self.reported_object
        ode_sys =system
        fun = system.dvars[0]
        y = fun
        ivar = system.ivar

        fun_str = system._variable_name()[3] #y'

        d_ivar = system._variable_name()[0] #dx
        d_var = system._variable_name()[1] #dy

        y_sym = system._variable_name()[2] #y(x)
        const=Symbol('D')

        display(ReportText('Przenosimy wszystkie elementy z funkcją $y$ i jej pochodnymi na jedną stronę i przyrównujemy do zera, otrzymując równanie jednorodne.'))
        #display(SympyFormula(Eq(system._hom_equation().lhs[0],0)))
        
        
        


class LinearToSeparable(ReportComponent):
    
    title="Rozwiązanie powstałego równania o zmiennych rozdzielonych"
    def append_elements(self):

        system = self.reported_object
        fun = system.dvars[0]
        
        ode_sys=system._hom_equation()
        #sep_ode_report=ode_sys.report #mniej więcej coś takiego tylko coś pomyliłem na 100%
        
        display(ReportText('Uzyskane równanie jednorodne jest równaniem o zmiennych rozdzielonych i przyjmuje postać:'))
        display(SympyFormula(Eq(ode_sys.lhs[0],0)))
        
        #print("#"*200)
        sep_ode_report=ode_sys.report
        
        
        #print(list(sep_ode_report))
        #print('#'*3)
        #print(list(sep_ode_report)[3:])
        
        for elem in list(sep_ode_report)[3:]:
            self.append(elem)



class VariationOfConstant(ReportComponent):
    title="Uzmiennienie stałej"
    
    def append_elements(self):

        system = self.reported_object
        ode_sys =system
        fun = system.dvars[0]
        y = fun
        ivar = system.ivar
        

        ode_sys2 = ode_sys
        x=ivar
        szur=ode_sys2.general_solution
#         display(szur)
        ode_syso=Eq(ode_sys2.lhs[0],ode_sys2.rhs[0])
        Cx = Function(szur._spot_constant()[0])(x)
        Cx
        c_prim=Function("C'")(x)
        szur2=szur.subs(szur._spot_constant()[0], Cx)
        #display(szur2)
        szur3=Eq(szur2.lhs.diff(x), szur2.rhs.diff(x))
        #display(szur3)
        szur3_=szur3.subs(Cx.diff(x),c_prim)
        szur3_
        #To implement, change left hand side to y_prim variable and subsitute variables with object variables

        ##### Tu sie teoretycznie zaczyna variation of constant ########################################
        sol_cx=(Eq(Symbol('y'),szur2.rhs[0]))
        display(ReportText('Kolejnym krokiem jest potraktowanie stałej jako funkcji zależnej od $x$:'))
        display(SympyFormula(sol_cx))
        sol_diff3=Eq(Symbol("y'"), szur3_.rhs[0])
        display(ReportText('Następnie należy policzyć pochodne obu stron równania'))
        ## dodać równanie 
        display(ReportText('Otrzymując równanie następującej postaci'))
                ##dodać spochodnione równanie chyba soldiff3 bedzie git
        
        
        display(SympyFormula(sol_diff3))#gdzie? gdzie co?
        display(ReportText("Dzięki tej operacji możemy teraz podstawić $y$ i $y'$ do oryginalnego równania"))
        display(SympyFormula(ode_syso))
        lolo=ode_syso.subs(y.diff(ivar),sol_diff3.rhs).subs(y,sol_cx.rhs)
        display(ReportText('I otrzymamy:'))
        display(SympyFormula(lolo))
        c_p_sol=solve(lolo,c_prim)
        cp_eq=Eq(c_prim,c_p_sol[0])
        display(ReportText("Następnie, jeżeli wszystko zostało policzne poprawnie to wartosci $C(x)$ powinny się skrócić i uprościć do postaci z której w łatwy sposób możemy wyznaczyć $C'(x)$ otrzymując:"))
        display(SympyFormula(cp_eq))
        cx_wyn=Eq(Cx,cp_eq.rhs.diff(ivar))
        display(ReportText("Po obustronnym scałkowaniu otrzymamy wartosć stałej:"))
        display(SympyFormula(cx_wyn))
        FINALOWY_WYNIK=sol_cx.subs(Cx,cx_wyn.rhs)
        display(ReportText("Aby otrzymać ostateczny wynik musimy podstawić otrzymaną stałą do pierwszego równania w którym uzmiennilismy stałą:"))
        display(SympyFormula(FINALOWY_WYNIK))



class BernoulliODEIntroComponent(ReportComponent):
    title="Podstawowa wiedza niezbędna do rozwiązania równania różniczkowego"
    
    def append_elements(self):

        system = self.reported_object
        y_sym = system.dvars[0]
        dane=system._get_dvars_symbols()
        Px=Symbol('P(x)')
        Qx=Symbol('Q(x)')
        n=Symbol('n')
        ynpow=Symbol('y')**Symbol('n')
        display(ReportText('Równanie różniczkowe typu Bernoulliego to szczególny rodzaj równania różniczkowego zwyczajnego pierwszego rzędu. Ma postać ogólną:'))
        bernoulli_form=Eq(y_sym.diff()+Px*y_sym,Qx*y_sym**n)
        display(SympyFormula(bernoulli_form.subs(dane)))
        
        display(ReportText(f'Gdzie $y$ to nieznana funkcja zależna od zmiennej $x$, $P(x)$ i $Q(x)$ o dane funkcje zależne od $x$, a $n$ to stała liczba rzeczywista, różna od 0 i 1. Charakterystyczne dla równań Bernoulliego jest to, że są one nieliniowe z powodu obecności wyrazu ${latex(ynpow)}$ po jednej stronie równania. Rozwiązania tego typu równań mogą być trudne do uzyskania w ogólności, ale dla pewnych wartości $n$ można zastosować pewne techniki upraszczające rozwiązanie. W przypadku $n=0$ równanie Bernoulliego staje się liniowe, a dla $n=1$ można je przekształcić do postaci liniowego równania różniczkowego zwyczajnego pierwszego rzędu. W innych przypadkach konieczne może być zastosowanie specjalnych technik, takich jak zamiana zmiennych czy redukcja rzędu, aby uzyskać ogólne rozwiązanie.'))

        

class BernoulliTransformation(ReportComponent):
    title=" Wyznaczenie nowej funkcji z zależnej od x w celu przejścia na równanie liniowe"
    
    def append_elements(self):
        system = self.reported_object
        ode_sys =system
        fun = system.dvars[0]
        ivar = system.ivar
###########################DO ZMIANYYYYYYYY########################
        fun_str = system._variable_name()[2] #y'

        d_ivar = system._variable_name()[0] #dx
        d_var = system._variable_name()[1] #dy
        dane=system._get_dvars_symbols()
        bernoulli_sub=system._bernoulli_substitution()
        #bern_power = system



        
        n=Symbol('n')
        z=Function('z')(ivar)

        display(ReportText('Aby dokonać podstawienia, konieczne jest zdefiniowanie $n$ poprzez znalezienie najwyższej potęgi $y$, gdzie $n$ to wykładnik tej funkcji. '))
        display(SympyFormula(Eq(Symbol('n'),n.subs(bernoulli_sub))))
        display(ReportText('Następnie należy dokonać podstawienia i przejść na nową funkcję $z$, używając poniższego wzoru:'))
        display(SympyFormula(Eq(Symbol('z'),Symbol('y')**(S.One-Symbol('n')))))
        display(ReportText('W naszym przypadku:'))
        display(SympyFormula(Eq(Symbol('z'),Symbol('y')**(S.One-n.subs(bernoulli_sub)))))
        #display(SympyFormula(Eq(Symbol('z'),z.subs(bernoulli_sub))))
        display(ReportText('Następnym krokiem jest wyznaczenie pierwszej pochodnej funkcji $z$:'))
        display(SympyFormula(Eq(Symbol("z'"), bernoulli_sub['z_diff'])))
        display(ReportText('Po podstawieniu wartosci n :'))
        display(SympyFormula(Eq(Symbol("z'"), bernoulli_sub['z_diff'].subs(n,bernoulli_sub[n]))))
        
        display(ReportText("Następnie należy wyznaczyć z powyższych równań $y$ oraz $y'$"))
        display(SympyFormula(Eq(Symbol("y"),fun.subs(bernoulli_sub))))
        display(SympyFormula(Eq(Symbol("y'"),fun.diff().subs(bernoulli_sub))))

class BernoulliLinearTransformation(ReportComponent):
    title=" Przejście na równanie liniowe"
    
    def append_elements(self):
        system = self.reported_object
        ode_sys =system
        fun = system.dvars[0]
        ivar = system.ivar
###########################DO ZMIANYYYYYYYY########################
        fun_str = system._variable_name()[2] #y'

        d_ivar = system._variable_name()[0] #dx
        d_var = system._variable_name()[1] #dy
        dane=system._get_dvars_symbols()

        rownanie=system._as_reduced_ode()

        display(ReportText("Ostatnim krokiem do przejscia na równanie liniowe jest podstawienie wyliczonych wyżej $y$ i $y'$ do pierwotnego równania:"))
        display(SympyFormula(Eq(rownanie.lhs[0],rownanie.rhs[0])))
        display(ReportText('Otrzymane równanie liczymy w następujący sposób:'))

        sep_ode_report=rownanie.report


        for elem in list(sep_ode_report)[3:]:
            self.append(elem)

            
            
            
            
#Karolina i Karol done         
            
general_code='''
from dynpy.models.odes.numerical import *
system=LinearFirstOrder.from_reference_data()
system.general_solution
'''            
class ODEGeneralSolutionComponent(ReportComponent):
    
    title="Wyznaczanie rozwiązania ogólnego";

    @property
    def import_text(self):

        return "Aby zaimportować przykładowy system  i wywołać rozwiązanie ogólne należy wywołać następujący kod:"


    @property
    def body_text(self):

        return "Rozwiązanie ogólne przyjmuje postać:"

    def append_elements(self):
        
        system = self.reported_object
        t=system.ivar
        dvars=system.dvars

        display(ReportText(self.import_text))
        display(ObjectCode(general_code))

        display(ReportText(self.body_text))

        display((SympyFormula(system.general_solution)))
        
steady_code='''
from dynpy.models.odes.numerical import *
system=LinearFirstOrder.from_reference_data()
system.steady_solution
'''        
class ODESteadySolutionComponent(ReportComponent):
    
    title="Wyznaczanie rozwiązania ogólnego";

    @property
    def import_text(self):

        return "Aby zaimportować przykładowy system  i wywołać rozwiązanie szczególne należy wywołać następujący kod:"


    @property
    def body_text(self):

        return "Rozwiązanie szczególne przyjmuje postać:"

    def append_elements(self):
        
        system = self.reported_object
        t=system.ivar
        dvars=system.dvars

        display(ReportText(self.import_text))
        display(ObjectCode(steady_code))

        display(ReportText(self.body_text))

        display((SympyFormula(system.steady_solution)))
        
import_code='''
from dynpy.models.odes.numerical import *

system=LinearFirstOrder.from_reference_data()
system.as_eq()  #zwraca równanie
system.as_matrix()  #zwraca macierz
system.as_iterable()  #zwraca listę
system.as_dict()  #zwraca słownik
system.as_eq_list()  #zwraca równanie w postaci listy
system._as_fode()  #zwraca równanie pierwszego rzędu

'''

class ODESystemRepresentationComponent(ReportComponent):
    
    title="Różnorodne reprezentacje systemu"

    @property
    def import_text(self):

        return "Metody umożliwiające różnorodne reprezentacje systemu powinny być wywoływane w następujący sposób:"


    def append_elements(self):

        system = self.reported_object
        t=system.ivar
        dvars=system.dvars

        display(ReportText(self.import_text))
        display(ObjectCode(import_code))

        display(ReportText(r'Metoda .as_eq()'))
        display((system.as_eq()))

        display(ReportText(r'Metoda .as_matrix()'))
        display((system.as_matrix()))

        display(ReportText(r'Metoda .as_iterable()'))
        display((system.as_iterable()))

        display(ReportText(r'Metoda .as_dict()'))
        display((system.as_dict()))

        display(ReportText(r'Metoda .as_eq_list()'))
        display((system.as_eq_list()))

        display(ReportText(r'Metoda ._as_fode()'))
        display((system._as_fode()))

class ODEInitCodeComponent(ReportComponent):
    
    title="Stworzenie liniowego równania różniczkowego"

    def append_elements(self):

        from dynpy.solvers.tools import CodePrinter

        system = self.reported_object
        ode_sys = system

        _dvars = system.dvars
        _odes = system.odes

        _ode_order = system.ode_order

        if _ode_order == 1:
            cls_name = 'FirstOrderODESystem'
        else:
            cls_name = 'ODESystem'

        string_gc = CodePrinter(_odes[0])._generate_code()

        display(ReportText(f'Stworzenie instacji klasy {cls_name} wymaga zdefiniowania następujących zmiennych oraz równań:'))

        display(ObjectCode(f'''


from dynpy.solvers.linear import *
from dynpy.models.odes.linear import *
from sympy import *

{string_gc}

dvars = {_dvars}
odes = {_odes}
ode_order = {_ode_order}

odesys = {cls_name}(odes = odes , dvars = dvars, ode_order = ode_order)

display(odesys)

        '''))

        display(ReportText(f'Po wykonaniu instrukcji mamy gotową instancję obiektu klasy {cls_name}'  ))
        
class ODESystemAdditionComponent(ODESystemOperations):
    
    title="Adding two ODESystem Matrix"

    def append_elements(self):
            from sympy import Symbol, Function, Matrix
            
            systems = self.reported_object
            l = len(systems)
            matrix_add = self.get_zero_ode_sys()
            
            matrix_mat_lhs = Matrix([0])
            matrix_mat_rhs = Matrix([0])
            
            count = 1
            display(ReportText('#### Sprawdzanie sumy macierzy:'))
            for ode_sys in systems:
                display(ReportText(f'Macierz {count}'))
                display(ode_sys)
                count += 1
                matrix_add += ode_sys
                
                matrix_mat_lhs += ode_sys.lhs
                matrix_mat_rhs += ode_sys.rhs
                
            display(ReportText('Suma macierzy ODEsystem'))
            display(matrix_add)
            display(ReportText('Suma macierzy Sympy'))
            display(Eq(matrix_mat_lhs, matrix_mat_rhs, evaluate=False))

            
class ODESystemSubtractionComponent(ODESystemOperations):
    
    title="Subtractiing two ODESystem Matrix"

    def append_elements(self):
            systems = self.reported_object
            l = len(systems)
            matrix_sub = systems[0]
            
            matrix_mat_lhs = systems[0].lhs
            matrix_mat_rhs = systems[0].rhs
            
            display(ReportText('#### Sprawdzanie różnicy macierzy:'))
            display(ReportText(f'Macierz 1'))
            display(systems[0])
            count = 2
            for ode_sys in systems[1:]:
                display(ReportText(f'Macierz {count}'))
                display(ode_sys)
                count += 1
                matrix_sub -= ode_sys
                
                matrix_mat_lhs -= ode_sys.lhs
                matrix_mat_rhs -= ode_sys.rhs
                
            display(ReportText('Różnica macierzy ODEsystem'))
            display(matrix_sub)
            display(ReportText('Różnica macierzy Sympy'))
            display(Eq(matrix_mat_lhs, matrix_mat_rhs, evaluate=False))
            
class ODESystemMultiplicationComponent(ODESystemOperations):
    
    title="Multiplying two ODESystem Matrix"

    def append_elements(self):
            systems = self.reported_object
            l = len(systems)
            matrix_mul = 1
            count = 1
            
            matrix_mat_lhs = Matrix([1])
            matrix_mat_rhs = Matrix([1])
            
            display(ReportText('#### Sprawdzanie iloczynu macierzy:'))
            for ode_sys in systems:
                display(ReportText(f'Macierz {count}'))
                display(ode_sys)
                count += 1
                matrix_mul *= ode_sys
                
                matrix_mat_lhs *= ode_sys.lhs
                matrix_mat_rhs *= ode_sys.rhs
                
            display(ReportText('Iloczyn macierzy ODEsystem'))
            display(matrix_mul)
            display(ReportText('Iloczyn macierzy Sympy'))
            display(Eq(matrix_mat_lhs, matrix_mat_rhs, evaluate=False))
            
class ODESystemExponentiationComponent(ReportComponent):
    title="Exponentiation of ODESystem Matrix"

    def append_elements(self):
        system = self.reported_object
        ode_sys = system
        
        display(ReportText('#### Sprawdzanie potęgi macierzy:'))
        display(ReportText('Macierz'))
        display(ode_sys)

        
        display(ReportText('Potęga macierzy ODEsystem'))
        
        matrix_exp = ode_sys * ode_sys
        display(matrix_exp)
        
        matrix_mat_lhs = ode_sys.lhs * ode_sys.lhs
        matrix_mat_rhs = ode_sys.rhs * ode_sys.rhs
        
        display(ReportText('Potęga macierzy Sympy'))
        display(Eq(matrix_mat_lhs, matrix_mat_rhs, evaluate=False))
        
excitation_code='''
from dynpy.models.odes.numerical import *
system=LinearFirstOrder.from_reference_data()
system._as_fode()._get_excitation_comps
'''    
        
class PredictionOfSteadySolutionComponent(ReportComponent):
    
    title="Wyznaczanie przewidywanego rozwiązania szczególnego równania";

    @property
    def import_text(self):

        return "Aby zaimportować przykładowy system  i wyznaczyć jego przewidywane rozwiązanie szczególne należy wywołać następujący kod:"


    @property
    def body_text(self):

        return "Rozwiązanie przyjmuje postać:"

    def append_elements(self):
        system = self.reported_object
        t=system.ivar
        dvars=system.dvars
        display(ReportText(self.import_text))
        display(ObjectCode(excitation_code))
        display(ReportText(self.body_text))
        system._as_fode()._get_excitation_comps

        ext_comps_list = system._as_fode()._get_excitation_comps

        for comp,amp in ext_comps_list:

            display(SympyFormula(comp))
            display(SympyFormula(amp))

class HomoPredictionIntroComponent(ReportComponent):

    title="Równanie różniczkowe drugiego rzędu - metoda przewidywań"
    def append_elements(self):
        system = self.reported_object
        dvar=system.dvars[0]
        ivar=system.ivar
        display(ReportText('Jednym z podstawowych typów równań różniczkowych drugiego rzędu jest równanie jednorodne o stałych współczynnikach . Proces rozwiązania tego typu równania polega na przewidzeniu rozwiązania, które przyjmuje ogólną postać:'))
        display(SympyFormula(Eq(dvar,Symbol("C")*exp(Symbol('r')*ivar))))
        display(ReportText('Na podstawie otrzymanych pierwiastków (rozwiązań) przewidywanego równania dobiera się docelowe równanie ogólne. Poniżej znajduje się przykład zadania ilustrującego proces rozwiązania tego rodzaju równania:'))


class MainPredictionComponent(ReportComponent):

    title="Przewidywanie rozwiązania równania jednorodnego"

    def append_elements(self):

        system = self.reported_object._hom_equation()
        C=Symbol("C",positive=True)
        r=Symbol("r")
        y=system.dvars[0]
        x=system.ivar
        sym_dict=system._get_dvars_symbols()
        sys_sub=Eq(system.lhs[0],system.rhs[0])

        prdct=Eq(y,C*exp(r*x))
        prdct_diff=Eq(prdct.lhs.diff(x),prdct.rhs.diff(x))
        prdct_ddiff=Eq(prdct.lhs.diff(x,2),prdct.rhs.diff(x,2))

        prdct_dict={y:prdct.rhs, y.diff(x):prdct_diff.rhs, y.diff(x,2):prdct_ddiff.rhs}

        r_eq=Eq(system.fundamental_matrix()[0],0)
        r0_eq=Eq(y.subs(sym_dict),Symbol("r^0"))
        r1_eq=Eq(y.diff(x).subs(sym_dict),Symbol("r^1"))
        r2_eq=Eq(y.diff(x,2).subs(sym_dict),Symbol("r^2"))

        display(ReportText("Pierwszym krokiem do rozwiązania danego równania jest wyznaczenie pochodnych przewidywanego rozwiązania. Przypominając pierwotne równanie które przewidujemy jako nasze rozwiązanie ma następującą postać:"))
        display(SympyFormula(prdct.subs(sym_dict)))
        display(ReportText("A następnie znalezienie potrzebnych pochodnych. W przypadku równań drugiego rzędu jak nazwa wskazuje najwyższa pochodna jest drugiego stopnia stąd potrzebne nam dwie pierwsze pochodne przewidywanego rozwiązania. Pierwsza pochodna:"))
        display(SympyFormula(prdct_diff.subs(sym_dict)))
        display(ReportText("Druga pochodna:"))
        display(SympyFormula(prdct_ddiff.subs(sym_dict)))
        display(ReportText("Następnym krokiem jest podstawienie poszczególnych przewidywań do odpowiednich pochodnych w pierwotnym równaniu:"))
        display(SympyFormula(sys_sub.subs(prdct_dict)))
        display(ReportText("Można zauważyć powtarzające się elementy i wyłączyć je przed nawias:"))
        display(SympyFormula(sys_sub.subs(prdct_dict).simplify()))
        display(ReportText("Podstawowa zasada matematyki mówi, że jeśli iloczyn dwóch czynników jest równy zero, to przynajmniej jeden z tych czynników musi być równy zero. W metodzie przewidywań nie rozpatruje się jednak dwóch przypadków, ponieważ gdy stała całkowania, a zatem funkcja eksponenty, jest równa zeru to zadanie staje się trywialne i nie wymaga dalszej analizy. Dlatego skupiamy się na drugim czynniku, powszechnie nazywanym równaniem charakterystycznym, które dostarczy nam pierwiastków niezbędnych do przewidzenia ostatecznego równania. Równanie poddawane dalszej analizie ma następująca postać:"))
        display(SympyFormula(r_eq))
        display(ReportText("Po rozwiązaniu pewnej liczby zadań można zauważyć pewną zależność wynikającą z powyższego przewidywania, co pozwoli na pominięcie tego kroku i podstawiania odpowiednich wartości zgodnie z poniższymi podstawieniami:"))
        display(SympyFormula(r0_eq))
        display(SympyFormula(r1_eq))
        display(SympyFormula(r2_eq))

class RootsAnalysisComponent(ReportComponent):

    title="Rozwiązanie równania charakterystycznego i ogólna analiza jego rozwiązania w kontekście rozwiązywania równań różniczkowych drugiego rzędu"

    def append_elements(self):

        system = self.reported_object._hom_equation()
        sym_dict=system._get_dvars_symbols()
        y=system.dvars[0].subs(sym_dict)
        x=system.ivar
        char=system.fundamental_matrix()[0]
        char_eq=Eq(char,0)
        roots=system._eigenvalues()
        C1=Symbol("C_1",positive=True)
        C2=Symbol("C_2",positive=True)
        r0=Symbol("r_0")
        r1=Symbol("r_1")
        a=Symbol("alpha")
        b=Symbol("beta")
        i=Symbol("i")

        r1_img=Eq(r0,a+i*b)
        r2_img=Eq(r1,a-i*b)

        pred_real=Eq(y,C1*exp(r0*x)+C2*exp(r1*x))
        pred_one=Eq(y,C1*exp(r0*x)+C2*x*exp(r0*x))
        pred_imagine=Eq(y,exp(a*x)*(C1*cos(b*x)+C2*sin(b*x)))

        sol_eq=Eq(y,system._ode_solution().rhs[0])

        display(ReportText(f"W równaniach różniczkowych drugiego rzędu równanie charakterystyczne przyjmuje postać równania kwadratowego. Takie równanie może w rozwiązaniu dać trzy typy pierwiastków i w zależności od danego typu dobiera się rozwiązanie ogólne tego równania. Gdy policzona delta jest większa od zera, otrzymujemy dwa pierwiastki rzeczywiste o postaci ${latex(r0)}$ i ${latex(r1)}$, a wtedy przewidywane rozwiązanie przyjmuje postać:"))
        display(SympyFormula(pred_real))
        display(ReportText(f"Gdy wyliczona delta jest równa 0, otrzymujemy jeden pierwiastek o postaci ${latex(r0)}$, a wtedy przewidywane rozwiązanie przyjmuje postać:"))
        display(SympyFormula(pred_one))
        display(ReportText("Gdy wyliczona delta jest mniejsza od 0, otrzymujemy dwa pierwiastki zespolone o postaci:"))
        display(SympyFormula(r1_img))
        display(SympyFormula(r2_img))
        display(ReportText(f"Gdzie ${latex(a)}$ to część rzeczywista pierwiastka, a ${latex(b)}$ to część urojona. Przewidywane rozwiązanie w takim wypadku przyjmuje następującą postać:"))
        display(SympyFormula(pred_imagine))
        display(ReportText("Równanie charakterystyczne w danym przykładzie ma postać:"))
        display(SympyFormula(char_eq))
        display(ReportText("Pierwiastki tego równania przyjmują postać:"))
        i=0
        lista=system._roots_list()
        for elem in lista:
            if elem !=0:
                display(SympyFormula( Eq(Symbol(f"r_{i}"),elem) , marker=None))
                i+=1
        display(ReportText("Po rozpoznaniu typu pierwiastków możemy zapisać rozwiązanie jako:"))
        display(SympyFormula(sol_eq))


from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset, atan, exp)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from dynpy.models.mechanics.principles import ComposedSystem
from dynpy.models.mechanics.disk import RollingDisk
from dynpy.models.electric.elements import *
from ..elements import GravitationalForce
#from dynpy.models.electric.engine import *
from functools import cached_property, lru_cache

from pint import UnitRegistry
ureg = UnitRegistry()

class ReducedMotorbike(ComposedSystem):
    scheme_name = 'DC_motor.png'

    M = Symbol('M', positive=True)
    m_b = Symbol('m_bike',positive=True)
    m_d = Symbol('m_driver',positive=True)
    m_r= Symbol('m_r',positive=True)
    m_f = Symbol('m_f',positive=True)
    r = Symbol('r',positive=True)
    Cd=Symbol('C_d',positive=True) #dragcoefficient
    Ad=Symbol('A_d',positive=True) #air density
    Af=Symbol('A_f',positive=True) #front area of the vehicle
    T=Symbol('T')
    f_t=Symbol("f_t")
    g=Symbol("g")

    x=dynamicsymbols('x')
    phi=dynamicsymbols('phi')

    def __init__(self,
                 M=None,
                 m_b=None,
                 m_d=None,
                 m_r=None,
                 m_f=None,
                 x=None,
                 r=None,
                 Cd=None,
                 Ad=None,
                 Af=None,
                 T=None,
                 g=None,
                 f_t=None,
                 phi=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if M is not None: self.M=M
        if m_b is not None: self.m_b = m_b
        if m_d is not None: self.m_d = m_d
        if m_r is not None: self.m_r = m_r
        if m_f is not None: self.m_f = m_f
        if x is not None: self.x = x
        if r is not None: self.r=r
        if Cd is not None: self.Cd=Cd
        if Ad is not None: self.Ad=Ad
        if Af is not None: self.Af=Af
        if T is not None: self.T=T
        if g is not None: self.g=g
        if f_t is not None: self.f_t=f_t
        if phi is not None: self.phi=phi

        self.ivar = ivar

        self.qs = [self.x]
        self._init_from_components(**kwargs)

    @property
    def linear_displacement(self):

        return self.x

    @property
    def angular_displacement(self):

        return ((self.x)/(self.r))

    @property
    def linear_velocity(self):

        return self.x.diff(self.ivar)

    @cached_property
    def components(self):
        components = {}

        self.mass_bike=MaterialPoint(self.m_b,self.linear_displacement,qs=self.qs,ivar=self.ivar)
        self.mass_driver=MaterialPoint(self.m_d,self.linear_displacement,qs=self.qs,ivar=self.ivar)
        self.rear_wheel=RollingDisk(self.m_r, R=self.r, x=self.x, qs=self.qs, ivar = self.ivar)(label='Rear wheel')
        self.front_wheel=RollingDisk(self.m_f, R=self.r, x=self.x, qs=self.qs, ivar = self.ivar)(label='Front wheel')
        self.engine_moment=Force(self.T,pos1 = self.angular_displacement,qs =self.qs,ivar=self.ivar)(label='Bike engine')
        self.drag_force=Force(-S.One/2*self.Ad*self.Cd*self.Af*(self.linear_velocity )**2,self.x,qs=self.qs,ivar=self.ivar)
        #self.gravitationalforce = GravitationalForce(self.m_b+self.m_d+self.m_f+self.m_r, self.g, pos1=self.linear_displacement, qs = self.qs)
        self.rolling_resistance=Force(-self.f_t*(self.m_b+self.m_d+self.m_f+self.m_r)*self.g,pos1 = self.linear_displacement,qs =self.qs,ivar=self.ivar)(label='Rolling resistance force')
        
        components['mass_bike'] = self.mass_bike
        components['mass_driver']=self.mass_driver
        components['rear_wheel']=self.rear_wheel
        components['front_wheel']=self.front_wheel
        components['engine_moment']=self.engine_moment
        components['drag_force']=self.drag_force
        #components['gravitational_force'] = self.gravitationalforce
        components['rolling_resistance'] = self.rolling_resistance


        return components

    def v_max(self):
        vmax=Symbol("v_{max}",positive=True)
        eq=Eq(self.eoms[0].subs(self.linear_velocity,vmax).doit(),0)
        sol=solve(eq,vmax)[1]
        return Eq(vmax,sol)
    
    def symbols_description(self):
        #i_roc=self.i_w.diff(self.ivar)
        vmax=Symbol("v_{max}",positive=True)
        self.sym_desc_dict = {
#             self.U_z: r'voltage supplying the rotor',
#             self.R_w: r'equivalent resistance of the rotor windings',
#             self.L_w: r'equivalent inductance of the rotor windings',
#             self.E: r'electromotive force of induction',
#             self.U_Rw: r'voltage across the rotor winding resistance',
#             self.U_Lw: r'voltage related to the rotor inductance',
#             self.k_e: r'back electromotive force constant',
#             self.M_s: r'rotor torque',
#             self.B: r'coefficient of viscous friction reduced to the rotor shaft',
#             self.J: r'moment of inertia reduced to the rotor shaft',
#             self.M_obc: r'engine load torque',
#             self.k_m: r'torque constant',
#             self.i_w: r'rotor winding current',
#             i_roc:r'rotor winding current rate of change',
#             self.omega_s:r'angular velocity of the rotor',
#             self.M_a:r'rotor angular acceleration torque',
#             self.M_r:r'rotor motion resistance torque',
#             self.omega_s.diff(self.ivar):r'angular acceleration of the rotor',
            self.m_b: r'Mass of the motorcycle',
            self.m_d: r'Mass of the driver ',
            self.r: r'Dynamic radius of the motorcycle',
            self.m_f: r'Mass of the front wheel',
            self.m_r: r'Mass of the rear wheel',
            self.x.diff(): r'Velocity of the system',
            self.x.diff(self.ivar,2): r'Acceleration of the system',
            self.ivar: r'Time',
            self.T: r'Torque',
            self.Cd: r'Drag coefficient',
            self.Ad: r'Air density',
            self.Af: r'Frontal area of the motorcycle',
            self.x: r'The displacement of the system',
            self.g: r'Gravitational acceleration',
            self.f_t: r'Rolling resistance coefficient',
            vmax: 'The maximum speed'
        }
        return self.sym_desc_dict

    def get_default_data(self):
        default_data_dict = {
#             self.U_z: [10],
#             self.R_w: [2],
#             self.L_w: [0.1],
#             self.k_e: [0.1],
#             self.k_m: [0.1],
#             self.J: [0.1],
#             self.B: [0.5],
#             self.M_obc: [0.2],
            self.r:[0.3175],
#             self.c:[1000],
            self.m_b:[270],
            self.m_d:[80],
            self.m_r:[19],
            self.m_f:[15],
            self.Cd:[0.65],
            self.Ad:[1.2047],
            self.Af:[0.6]
        }
        return default_data_dict


    def _dimensionless_ode(self):

        from ...solvers.linear import ODESystem,FirstOrderLinearODESystem

        t = self.ivar
        tau = Symbol('tau')

        syms_dict = {
                    # dcmotor.U_z: 0,
                    self.R_w: 1,
                    self.L_w: 1,
                    # dcmotor.k_e: Symbol('kappa',positive=True),
                    # dcmotor.k_m: Symbol('kappa',positive=True),

                    self.k_e: Symbol('kappa',negative=True),
                    self.k_m: Symbol('kappa',negative=True),

                    self.J: 1,
                    self.B: 1,
                    t:tau,
                    # dcmotor.M_obc: 10,
                    }
        dvars = Matrix([[self.i_w], [self.omega_s]]).subs({t:tau})

        ode = ODESystem(odes=(self._eoms.subs(syms_dict))*-1, dvars = dvars, ode_order=1,ivar=tau)

        #return FirstOrderLinearODESystem.from_ode_system(ode)
        return ode

    def atan_torque(self,k=1,steady_val=3500):

        torque_expr=(k*(1/2 - (1 / 3.14) * atan((60*self.x.diff(self.ivar)/(2*pi.n()*self.r)) - steady_val)))


        return self.subs(self.T,torque_expr)
    def units(self):
        
        units_dict={
            self.m_b: ureg.kilogram,
            self.m_d: ureg.kilogram,
            self.r: ureg.meter,
            self.m_f: ureg.kilogram,
            self.m_r: ureg.kilogram,
            self.ivar: ureg.second,
            self.T: ureg.newton*ureg.meter,
            self.Ad: ureg.kilogram,
            self.Af: ureg.meter**2,
            self.x: ureg.meter,
            self.x.diff(self.ivar): ureg.meter/ureg.second,
            self.x.diff(self.ivar,2): ureg.meter/ureg.second**2,
            self.g:ureg.meter/ureg.second**2,
            
           }
        return units_dict

class MotorbikeReducedToDisk(ReducedMotorbike):

    phi=dynamicsymbols('\phi')

    def __init__(self,
             M=None,
             m_b=None,
             m_d=None,
             m_r=None,
             m_f=None,
             x=None,
             r=None,
             Cd=None,
             Ad=None,
             Af=None,
             T=None,
             phi=None,
             ivar=Symbol('t'),
             **kwargs):

        if M is not None: self.M=M
        if m_b is not None: self.m_b = m_b
        if m_d is not None: self.m_d = m_d
        if m_r is not None: self.m_r = m_r
        if m_f is not None: self.m_f = m_f
        if x is not None: self.x = x
        if r is not None: self.r=r
        if Cd is not None: self.Cd=Cd
        if Ad is not None: self.Ad=Ad
        if Af is not None: self.Af=Af
        if T is not None: self.T=T
        if phi is not None: self.phi=phi

        self.ivar = ivar

        self.qs = [self.phi]
        self._init_from_components(**kwargs)

    @property
    def linear_displacement(self):

        return self.phi*self.r

    @property
    def angular_displacement(self):

        return self.phi

    @property
    def linear_velocity(self):

        return (self.phi*self.r).diff(self.ivar)

    @cached_property
    def components(self):
        components = {}

        self.mass_bike=MaterialPoint(self.m_b,self.linear_displacement,qs=self.qs,ivar=self.ivar)
        self.mass_driver=MaterialPoint(self.m_d,self.linear_displacement,qs=self.qs,ivar=self.ivar)
        self.rear_wheel=RollingDisk(self.m_r, R=self.r, x=self.linear_displacement, qs=self.qs, ivar = self.ivar)(label='Rear wheel')
        self.front_wheel=RollingDisk(self.m_f, R=self.r, x=self.linear_displacement, qs=self.qs, ivar = self.ivar)(label='Front wheel')
        self.engine_moment=Force(self.T,pos1 = self.angular_displacement,qs =self.qs,ivar=self.ivar)(label='Bike engine')
        self.drag_force=Force(-S.One/2*self.Ad*self.Cd*self.Af*(self.linear_displacement.diff(self.ivar))**2,self.phi,qs=self.qs,ivar=self.ivar)

        components['mass_bike'] = self.mass_bike
        components['mass_driver']=self.mass_driver
        components['rear_wheel']=self.rear_wheel
        components['front_wheel']=self.front_wheel
        components['engine_moment']=self.engine_moment
        components['drag_force']=self.drag_force


        return components


    def omega_max(self):
        omega_max=Symbol("\omega_{max}")
        eq=Eq(self.eoms[0].subs(self.angular_displacement.diff(self.ivar),omega_max).doit(),0)
        sol=solve(eq,omega_max)
        return Eq(omega_max,sol[1])


import importlib
from . import dynamics as dyn
importlib.reload(dyn)
from . import dynamics as dyn

from sympy.physics.vector import dynamicsymbols
from sympy import *
from sympy.physics.mechanics import *
import sympy as sym
import numpy as np
import pylatex
from pylatex import Command, NoEscape
import pint
from dynpy.dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp
from dynpy.models.elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame,base_origin
from dynpy.models.mechanics.trolley import SpringMassSystem, SpringDamperMassSystem
# from dynpy.models.sdof import ComposedSystem, SpringMassSystem, DampedSpringMassSystem, BeamBridgeDamped
ureg = pint.UnitRegistry()
from dynpy.models.mechanics.principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin
mechanics_printing()



t=Symbol('t') #independent variable - time

class LEWEL16_simple(SpringMassSystem):
    scheme_name = 'train_sdof.png'
    real_name = 'lewel16.jpg'

class LEWEL16_simple_damped(SpringDamperMassSystem):
    scheme_name = 'train_sdof_damped.png'
    real_name = 'lewel16.jpg'

class LeafSpring(ComposedSystem):

    m=Symbol('m', positive=True)
    k_beam=Symbol('k_beam', positive=True)
    ivar=Symbol('t')
    g=Symbol('g', positive=True)
    l=Symbol('l', positive=True)
    n=Symbol('n',positive=True)
    t=Symbol('s',positive=True)
    w=Symbol('w',positive=True)
    module=Symbol('E', positive=True)
    z=dynamicsymbols('z')
    
    scheme_name = 'bridge_dmp.png'
    real_name = 'leaf_spring.jpg'
    
    def __init__(self,
                 m=None,
                 k_beam=None,
                 ivar=None,
                 g=None,
                 l=None,
                 n=None,
                 t=None,
                 w=None,
                 module=None,
                 z=None,
                 **kwargs):

        if m is not None: self.m = m
        if k_beam is not None: self.k_beam = k_beam
        if g is not None: self.g = g
        if l is not None: self.l=l
        if n is not None: self.n=n
        if t is not None: self.t=t
        if w is not None: self.w=w
        if z is not None: self.z = z
        if module is not None: self.module=module


        
        self.qs=[self.z]
        self._init_from_components(**kwargs)


        
#         self.force = Force(-F_0 * sin(Omega * ivar), pos1=z)



    @property
    def components(self):

        components = {}

        self._mass = MaterialPoint(self.m, self.z, qs=self.qs)
        self._spring = Spring(self.k_beam, self.z, qs=self.qs)
        self._gravitational_force = GravitationalForce(self.m, self.g, self.z, qs=self.qs)
        components['_mass'] = self._mass
        components['_spring'] = self._spring
        components['_gravitational_force'] = self._gravitational_force

        return components


    def set_equivalent_data(self):
        #E - Young's modulus, n - number of leaves, w - width of leaves, t - thickness of leaves, l - span; https://www.piping-designer.com/index.php/disciplines/mechanical/stationary-equipment/fastener/2486-leaf-spring-stiffness

        equivalent_data_dict = {
            self.k_beam: (8*self.module*self.n*self.w*self.t**3)/(3*self.l**3),
        }

        return equivalent_data_dict
    
    def get_default_data(self):
        default_data_dict = {self.m:3000,self.module:2.1*10**11,self.l:0.9,self.g:9.81,self.n:8,self.t:0.0135,self.w:0.08
        }

        return default_data_dict
    def get_real_data(self):

        real_data_dict = {self.m:3000,self.module:2.1*10**11,self.l:0.9,self.g:9.81,self.n:8,self.t:0.0135,self.w:0.08
        }

        return real_data_dict
    
class DDoFTrain(ComposedSystem):

    m=Symbol('m', positive=True)
    I=Symbol('I',positive=True)
    k=Symbol('k', positive=True)
    c=Symbol('c', positive=True)
    l_l=Symbol('l_l', positive=True)
    l_r=Symbol('l_r', positive=True)
    w=Symbol('w', positive=True)
    k_t=Symbol('k_t',positive=True)
#     Omega1=Symbol('Omega1',postive=True)
    Omega2=Symbol('Omega2',postive=True)
    S=Symbol('S',postive=True)
#     A=Symbol('A',postive=True)
    lam=Symbol('lambda',postive=True)
    ivar=Symbol('t')
    s=Symbol('s',positive=True)
    v=Symbol('v',positive=True)
    d=Symbol('d',positive=True)
    h=Symbol('h', positive=True)
    l_0=Symbol('l_0', positive=True)
    f=Symbol('f', positive=True)
#Dynamic Symbols
    z=dynamicsymbols('z')
    phi=dynamicsymbols('varphi')
    

    def __init__(self,
                     m=None,
                     I=None,
                     k=None,
                     c=None,
                     l_l=None,
                     l_r=None,
                     w=None,
#                      Omega1=None,
                     Omega2=None,
                     S=None,
#                      A=None,
                      lam=None,
                     z=None,
                     phi=None,
                     s=None,
                     v=None,
                     d=None,
                     h=None,
                     k_t=None,
                     l_0=None,
                     f=None,
                     ivar=Symbol('t'),
                     **kwargs):



            if m is not None: self.m = m
            if I is not None: self.I = I
            if k is not None: self.k = k
            if c is not None: self.c = c
            if l_l is not None: self.l_l = l_l
            if l_r is not None: self.l_r = l_r
            if w is not None: self.w = w
#             if Omega1 is not None: self.Omega1 = Omega1
            if Omega2 is not None: self.Omega2 = Omega2
            if S is not None: self.S = S
#             if A is not None: self.A = A
            if lam is not None: self.lam = lam
            if z is not None: self.z = z
            if phi is not None: self.phi= phi
            if s is not None: self.s= s
            if v is not None: self.v= v
            if d is not None: self.d= d
            if h is not None: self.h= h
            if k_t is not None: self.k_t=k_t
            if l_0 is not None: self.l_0=l_0
            if f is not None: self.f=f
            self.ivar = ivar

            self.qs = [self.z,self.phi]

            self._init_from_components(**kwargs)

    @property
    def components(self):

            components = {}

            self._train_body = RigidBody2D(self.m,
                                           self.I,
                                           pos_lin=self.z,
                                           pos_rot=self.phi,
                                           qs=self.qs)(label='body')
#             self._harmonic_force = Force(self.A*sin(self.Omega1*self.ivar),
#                                           pos1=self.phi,
#                                           qs=self.qs)(label='harmonic force')


#             self._spring_left = Spring(self.k,
#                                  pos1=self.z + self.phi*self.l_l,
#                                        pos2=Heaviside(self.ivar-50)*self.S1*sin(self.Omega2*(self.ivar-self.t0)),
#                                   qs=self.qs)(label='left spring')

#             self._spring_right = Spring(self.k,
#                                  pos1=self.z - self.phi*self.l_r,
#                                         pos2=Heaviside(self.ivar-50)*self.S2*sin(self.Omega2*self.ivar),
#                                   qs=self.qs)(label='right spring')

#             self._damper_left = Damper(self.c,
#                                        pos1= self.z + self.phi*self.l_l,
#                                        pos2=Heaviside(self.ivar-50)*self.S1*sin(self.Omega2*(self.ivar-self.t0)),
#                                        qs=self.qs) (label = 'left damper')

#             self._damper_right = Damper(self.c,
#                                        pos1= self.z - self.phi*self.l_r,
#                                        pos2=Heaviside(self.ivar-50)*self.S2*sin(self.Omega2*self.ivar),
#                                        qs=self.qs) (label = 'right damper')
####################################################### Wymuszneie zwykłe ##################################################################
#             self._spring_left = Spring(self.k,
#                                  pos1=self.z + self.phi*self.l_l ,
#                                        pos2=self.S1*sin(self.Omega2*(self.ivar-self.s/self.v)), #i tak samo niżej W TŁUMIKACH TEŻ ;0
#                                  qs=self.qs)(label='left spring')

#             self._spring_right = Spring(self.k,
#                                  pos1=self.z - self.phi*self.l_r,
#                                         pos2=self.S1*sin(self.Omega2*(self.ivar)),
#                                   qs=self.qs)(label='right spring')

#             self._damper_left = Damper(self.c,
#                                        pos1= self.z + self.phi*self.l_l,
#                                        pos2=self.S1*sin(self.Omega2*(self.ivar-self.s/self.v)),
#                                        qs=self.qs) (label = 'left damper')

#             self._damper_right = Damper(self.c,
#                                        pos1= self.z - self.phi*self.l_r,
#                                        pos2=self.S1*sin(self.Omega2*(self.ivar)),
#                                        qs=self.qs) (label = 'right damper')

####################################################### Wymuszneie zależne od prędkości ########################################################

            self._spring_left = Spring(self.k,
                                 pos1=self.z + self.phi*self.l_l ,
                                       pos2=self.S*sin(2*pi*self.v/self.d*(self.ivar-self.s/self.v)), 
                                  qs=self.qs)(label='left spring')


            self._spring_right = Spring(self.k,
                                 pos1=self.z - self.phi*self.l_r,
                                        pos2=self.S*sin(2*pi*self.v/self.d*(self.ivar)),
                                  qs=self.qs)(label='right spring')

            self._damper_left = Damper(self.c,
                                       pos1= self.z + self.phi*self.l_l,
                                       pos2=self.S*sin(2*pi*self.v/self.d*(self.ivar-self.s/self.v)),
                                       qs=self.qs) (label = 'left damper')


            self._damper_right = Damper(self.c,
                                       pos1= self.z - self.phi*self.l_r,
                                       pos2=self.S*sin(2*pi*self.v/self.d*(self.ivar)),
                                       qs=self.qs) (label = 'right damper')
            
#######################################################TORSIONAL I HORIZONTAL########################################################
            
#             self._torsional_spring = Spring(self.k_t,
#                                  pos1=self.z/self.l_r,
# #                                         pos2=self.S1*sin(2*pi*self.v/self.d*(self.ivar)),
#                                   qs=self.qs)(label='torsional spring')
#            self._horizontal_spring = Spring(self.k_t,
#                                 pos1=((self.z +self.phi*self.l_l)**2+self.l_0**2)-self.l_0 ,
#                                         pos2=self.S1*sin(2*pi*self.v/self.d*(self.ivar)),
#                                  qs=self.qs)(label='torsional spring')


            components['_train_body'] = self._train_body
            components['_spring_left'] = self._spring_left
            components['_spring_right'] = self._spring_right
            components['_damper_left'] = self._damper_left
            components['_damper_right'] = self._damper_right
#            components['_horizontal_spring']=self._horizontal_spring
#             components['_harmonic_force'] = self._harmonic_force


            return components

    def symbols_description(self):
        self.sym_desc_dict={
            self.z: r'przemieszczenie pionowe nadwozia',
            self.phi: r'przemieszczenie kątowe nadwozia',
            self.phi.diff(self.ivar):r'prędkość kątowa nadwozia',
            self.z.diff(self.ivar):r'prędkość pionowa nadwozia',
            self.phi.diff(self.ivar, self.ivar):r'przyspieszenie kątowe nadwozia',
            self.z.diff(self.ivar, self.ivar):r'przyspieszenie pionowe nadwozia',
            self.m: r'masa modelu połówkowego',
            self.I: r'moment bezwładności modelu połówkowego',
            self.k: r'sztywność zastępcza usprężynowania pierwszego i drugiego stopnia',
            self.c: r'tłumienie zastępcze zestawów tłumiących pierwszego i drugiego stopnia',
            self.l_r: r'odległość układu sprężysto-tłumiącego po stronie prawej od środka obrotu',
            self.l_l: r'odległość układu sprężysto-tłumiącego po stronie lewej od środka obrotu',
            self.v:r'pozioma prędkość pojazdu',
            self.d: r'długość nierówności powstającej na szynie',
            self.S: r'amplituda wymuszenia kinematycznego',
            self.s: r'odległość pomiędzy wózkami jezdnymi',
            self.ivar: r'czas',
            self.w: r'długość modelu połówkowego',
            self.h: r'wysokość modelu połówkowego',
        }

        return self.sym_desc_dict



    def get_default_data(self):



        default_data_dict = {
            self.lam: S.One * self.lam,
            self.k: S.One * self.k,
            self.c: S.One * self.k * self.lam,
#             self.c: S.One * self.c,
            self.m: S.One * self.m,
            self.I: S.One * self.I,
            self.l_l: S.One * self.l_l,
            self.l_r: S.One * self.l_r,
            self.w : S.One * self.w,
#            self.Omega1: S.One * self.Omega1,
            self.Omega2: S.One * self.Omega2,
            self.S: S.One * self.S,
#            self.A: S.One * self.A,
            self.s: S.One * self.s,
            self.v: S.One * self.v,
            self.d: S.One * self.d,
            self.h: S.One * self.h,
            self.f: S.One * self.f,


        }
        return default_data_dict

    def get_numerical_data(self):

            
        default_data_dict = {
            self.c : 15000,
#            self.lam: 0.01,
            self.m : 11025,
            self.k: 500000,
            self.l_l : 3.063,
            self.l_r : 2.663,
            self.w : 6.125,
#             self.Omega1 : 3.14*10,
            self.Omega2 : 3.14*10,
            self.S : 0.01,
#             self.A : 0.1,
            self.s : 5.725,
            self.v : 16.6, #60 km/h prędkość przejazdów
            self.d : 30, #Zakres z artykułu 40-200m
            self.h : 1.8, #Środek cięzkości w środku geometrycznym
            self.I : self.m * (self.w**2 + self.h**2)/12,
            
        
            

        }
        return default_data_dict
    
    def default_data_table(self):
        default_data_dict = {
            self.c : 15000,
#            self.lam: 0.01,
            self.m : 11025,
            self.k: 500000,
            self.l_l : 3.063,
            self.l_r : 2.663,
            self.w : 6.125,
#             self.Omega1 : 3.14*10,
            self.Omega2 : 3.14*10,
            self.S : 0.01,
#             self.A : 0.1,
            self.s : 5.725,
            self.v : 16.6, #60 km/h prędkość przejazdów
            self.d : 30, #Zakres z artykułu 40-200m
            self.h : 1.8, #Środek cięzkości w środku geometrycznym
            self.I : self.m * (self.w**2 + self.h**2)/12,
            
        
            

        }
        default_data_dict[self.I]=(self.m * (self.w**2 + self.h**2)/12).subs(DDoFTrain().get_numerical_data()).n(3)
        return default_data_dict
    def unit_registry(self):
        
        unit_dict = {
            self.phi: ureg.rad,
            self.z : ureg.meter,
            self.z.diff(DDoFTrain.ivar,2) : ureg.meter/ureg.sec**2,
            self.phi.diff(DDoFTrain.ivar,2) : ureg.rad/ureg.sec**2,
            self.c : ureg.newton*ureg.sec/ureg.meter,
            self.m : ureg.kilogram,
            self.I : ureg.kilogram*ureg.meter**2,
            self.k : ureg.newton/ureg.meter,
            self.l_l : ureg.meter,
            self.l_r : ureg.meter,
#            self.Omega1 : ureg.rad/ureg.sec,
            self.Omega2 : ureg.rad/ureg.sec,
            self.S : ureg.meter,
#            self.A : ureg.meter,
            self.lam : ureg.sec,
            self.s : ureg.meter,
            self.v : ureg.meter/ureg.sec,
            self.d : ureg.meter,
            self.h : ureg.meter,
            self.f : ureg.hertz,
            self.w : ureg.meter,
        }
        return unit_dict

    def resonance_velocities(self):
        
        self.v_z1= self.natural_frequencies()[0]*self.d/(2*pi)
        self.v_z2=self.natural_frequencies()[3]*self.d/(2*pi)
        display(self.v_z1.subs(self.get_numerical_data()),self.v_z2.subs(self.get_numerical_data()))
        return (self.v_z1.subs(self.get_numerical_data()),self.v_z2.subs(self.get_numerical_data()))


from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S, diag, Eq,
                   hessian, Function, flatten, Tuple, im, pi, latex, dsolve,
                   solve, fraction, factorial,Subs, Number, oo, Abs, N)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ..dynamics import LagrangesDynamicSystem, HarmonicOscillator

from .elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame,base_origin
from ..continuous import ContinuousSystem, PlaneStressProblem



import base64
import random
import IPython as IP
import numpy as np

import inspect



from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin, Element
from  ..continuous import ContinuousSystem, PlaneStressProblem

import base64
import random
import IPython as IP
import numpy as np
import inspect

from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST


import pint
ureg = pint.UnitRegistry()

#TODO
#ISSUE #149
class VesselWithHeaveAndRotation(ComposedSystem):

    scheme_name = 'vessel.jpg'
    
    m_vessel=Symbol('M_vessel', positive=True)
    
    I_5=Symbol('I_5', positive=True)
    
    wave_level=dynamicsymbols('W')
    wave_slope=dynamicsymbols('S')
    rho=Symbol('rho', positive=True)
    g=Symbol('g', positive=True)
    A_wl=Symbol('A_wl',positive=True)
    V=Symbol('V', positive=True)
    GM_L=Symbol('GM_L', positive=True)
    CoB=Symbol('CoB', positive=True)
    CoF=Symbol('CoF', positive=True)
    A_h=Symbol('A_h',positive=True)
    Phi_h=Symbol('Phi_h',positive=True)
    omega=Symbol('omega',positive=True)
    
    H = dynamicsymbols('H')
    Phi = dynamicsymbols('Phi')
    qs=dynamicsymbols('H, Phi')
    ivar=Symbol('t')
    
    def __init__(self,
                 m_vessel=None,
                 I_5=None,

                 wave_level=None,
                 wave_slope=None,
                 rho=None,
                 g=None,
                 A_wl=None,
                 V=None,
                 GM_L=None,
                 CoB=None,
                 CoF=None,
                 A_h=None,
                 Phi_h=None,
                 omega=None,
                 qs=None,
                 H=None,
                 Phi=None,
                 
                 ivar=Symbol('t'),
                 **kwargs):

        if m_vessel is not None:self.m_vessel = m_vessel
        if I_5 is not None:self.I_5 = I_5
        if wave_level is not None:self.wave_level = wave_level
        if wave_slope is not None:self.wave_slope = wave_slope
        if rho is not None:self.rho = rho
        if g is not None:self.g = g
        if A_wl is not None:self.A_wl = A_wl
        if V is not None:self.V = V
        if GM_L is not None:self.GM_L = GM_L
        if CoB is not None:self.CoB = CoB
        if CoF is not None:self.CoF = CoF
        if A_h is not None:self.A_h = A_h
        if Phi_h is not None:self.Phi_h = Phi_h
        if omega is not None:self.omega = omega
        if H is not None:self.H = H
        if Phi is not None:self.H = Phi
        
        self.ivar = ivar

        
        
        self.M_matrix = Matrix([[self.m_vessel, 0], [0, self.I_5]]) #wszedzie teraz pobiera None # dopisz wszędzie self tut
        self.K_matrix = Matrix([[self.rho * self.g * self.A_wl, -self.rho * self.g * self.A_wl * (self.CoF - self.CoB)],#trzeba ify popsać? #ify jużsą tylko na to H i Phi brakue chyba
                           [-self.rho * self.g * self.A_wl * (self.CoF - self.CoB),
                            self.rho * self.g * self.V * self.GM_L]])
        
        if qs is not None: self.qs = qs
        self.qs = [self.H, self.Phi]
        

        
        # vessel mass and stiffness matrix
        #M_matrix = Matrix([[m_vessel, 0], [0, I_5]])

        #K_matrix = Matrix([[rho * g * A_wl, -rho * g * A_wl * (CoF - CoB)],
                           #[-rho * g * A_wl * (CoF - CoB),
                            #rho * g * V * GM_L]])

        # generalized displacements and velocities
        
        #wave_level = A_h*cos(omega*self.ivar + Phi_h)
        #wave_slope = diff(wave_level,self.ivar)*(omega/g)
        
        #dh_wave_slope = diff(wave_level,t,t)*(omega/g)   # TU JEST TEN MAGICZNY FORMUŁ, KTÓRY CHYBA JEDNAK POWINNIŚMY WPROWADZIĆ BO dq go tak nie policzy XD #
        
        q = Matrix(self.qs) + Matrix([self.wave_level, self.wave_slope])
        dq = q.diff(self.ivar)
        
        

        # lagrangian components definition
        
        self.kinetic_energy = S.Half * sum(dq.T * self.M_matrix * dq)
        self.potential_energy = S.Half * sum(Matrix(self.qs).T * self.K_matrix * Matrix(self.qs))
        
        self._init_from_components(**kwargs)
        
    @property
    def components(self):

        components = {}

        self._vessel_inertia = Element(self.kinetic_energy,
                                 qs=self.qs)
        self._buoyancy_stiffness = Element(-self.potential_energy,
                                 qs=self.qs)

        components['_vessel_inertia']=self._vessel_inertia
        components['_buoyancy_stiffnes']=self._buoyancy_stiffness

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m_vessel: r'mass of vessel,',
            self.I_5: r'moment of inertia of 5-th degree (with respect to y axis, determined by the radius of gyration),',
            self.q: r'vessel generalized coordinates,',
            self.wave_level: r'wave level,',
            self.wave_slope: r'wave slope,',
            self.rho: r'fluid density,',
            self.g: r'acceleration of gravity,',
            self.A_wl: r'wetted area,',
            self.V: r'submerged volume of the vessel,',
            self.GM_L: r'longitudinal metacentric height,',
            self.CoB: r'centre of buoyancy,',
            self.CoF: r'centre of floatation.',

        }

        return self.sym_desc_dict
    
    def numerical_data(self):
        numbers_dict={
            self.I_5:21*self.m_vessel,
            self.m_vessel: 1e7,
            self.wave_level: 0.5 * cos(2/7 * pi * self.ivar),
            self.wave_slope: 0.5 * cos(2/7 * pi * self.ivar),
            self.rho:1025,
            self.g: 9.81,
            self.A_wl:4025,
            self.V:25000,
            self.GM_L:290,
            self.CoB:3.6,
            self.CoF:13,
            self.A_h:0.45,
            self.Phi_h:-2.95,
        }
        
        return numbers_dict
    
    def get_numerical_parameters(self):

        m_vessel, I_5, g, rho, A_wl, V, GM_L, CoB, CoF, A_h, Phi_h, wave_level, wave_slope = self.m_vessel, self.I_5, self.g, self.rho, self.A_wl, self.V, self.GM_L, self.CoB, self.CoF, self.A_h, self.Phi_h, self.wave_level, self.wave_slope
        
        get_numerical_parameters = {

            self.I_5:21*self.m_vessel,
            self.m_vessel: 1e7,
            self.wave_level: 0.5 * cos(2/7 * pi * self.ivar),
            self.wave_slope: 0.5 * cos(2/7 * pi * self.ivar),
            self.rho:1025,
            self.g: 9.81,
            self.A_wl:4025,
            self.V:25000,
            self.GM_L:290,
            self.CoB:3.6,
            self.CoF:13,
            self.A_h:0.45,
            self.Phi_h:-2.95, 
        }
        return get_numerical_parameters
    
    def units(self):
        units_dict={

            self.q[1]:ureg.radian,
            self.q[0]:ureg.meter,
            self.q[1].diff(self.ivar):ureg.radian/ureg.second,
            self.q[0].diff(self.ivar):ureg.meter/ureg.second,

           }
        
        return units_dict    
    
#TODO
#ISSUE #150
class TDoFCompensatedPayload(ComposedSystem):

    scheme_name = '3dofs_new.PNG'

    def __init__(self,
                 phi=dynamicsymbols('varphi'),
                 h=dynamicsymbols('h'),
                 h_c=dynamicsymbols('h_c'),
                 m_p=Symbol('m_p', positive=True),
                 k_w=Symbol('k_w', positive=True),
                 l_0=Symbol('l_0', positive=True),
                 qs=dynamicsymbols('varphi h h_c'),
                 y_e=dynamicsymbols('y_e'),
                 z_e=dynamicsymbols('z_e'),
                 m_c=Symbol('m_c', positive=True),
                 k_c=Symbol('k_c', positive=True),
                 l_c=Symbol('l_c', positive=True),
                 l_w=Symbol('l_w', positive=True),
                 g=Symbol('g', positive=True),
                 h_eq=Symbol('h_eq', positive=True),
                 h_ceq=Symbol('h_ceq', positive=True),
                 ivar=Symbol('t'),
                 **kwargs):

        self.m_p = m_p
        self.k_w = k_w
        self.l_0 = l_0
        self.y_e = y_e
        self.z_e = z_e
        self.m_c = m_c
        self.k_c = k_c
        self.l_c = l_c
        self.l_w = l_w
        self.g = g
        self.h_eq = h_eq
        self.h_ceq = h_ceq
        self.ivar = ivar
        self.phi = phi
        self.h = h
        self.h_c = h_c
        self.qs = qs

        phi, h, h_c = qs

        y = (h + h_eq + l_0 + l_c) * sin(phi) + y_e
        z = (h + h_eq + l_0 + l_c) * cos(phi) + z_e

        y_c = (h_c + h_ceq + l_0) * sin(phi) + y_e
        z_c = (h_c + h_ceq + l_0) * cos(phi) + z_e

        y, z, y_c, z_c = dynamicsymbols('y_{p},z_{p},y_c,z_c')

        v_c = sqrt(diff(y_c, ivar)**2 + diff(z_c, ivar)**2)
        v = sqrt(diff(y, ivar)**2 + diff(z, ivar)**2)

        positions_dict = {
            y: (h + h_eq + l_0 + l_c) * sin(phi) + y_e,
            z: (h + h_eq + l_0 + l_c) * cos(phi) + z_e,
            y_c: (h_c + h_ceq + l_0) * sin(phi) + y_e,
            z_c: (h_c + h_ceq + l_0) * cos(phi) + z_e,
        }

        self.kinetic_energy = S.Half * m_p * v**2 + S.Half * m_c * v_c**2

        self.potential_energy = (S.Half * k_w * (h_c + h_ceq)**2 +
                                 S.Half* k_c * (h + h_eq - (h_c + h_ceq))**2 -
                                 m_p * g * z - m_c * g * z_c)

        super(HarmonicOscillator,self).__init__((self.kinetic_energy - self.potential_energy).subs(positions_dict).doit(),
                         qs=qs,
                         ivar=ivar,**kwargs)


    def single_dof_transformation(self,p=None):
        
        if p is None:
            p = S.One
        
        
        omega, delta, eps = symbols('omega, delta, varepsilon',positive=True)
        A_y, A_z, Phi_y, Phi_z = symbols('A_y, A_z, Phi_y, Phi_z,',positive=True)
        
        sdof_payload = self(self.phi)
        
        nonlin_mathieu_model = sdof_payload.subs({self.h:0,self.h_c:0,self.m_c:0}).subs({self.h_eq:self.l_c,self.h_ceq:self.l_c,self.l_0:self.l_w-self.l_c})

        mathieu_nonlin_approx = nonlin_mathieu_model.approximated(n=3)

        mathieu_acc_form = mathieu_nonlin_approx._to_acc()

        mathieu_basis = mathieu_acc_form.subs({self.l_c:0,
                                               self.g:omega**2*self.l_w})

        mathieu_instability_eq = mathieu_basis.subs(S.Half*omega**2*self.phi**2,(S.Half)**3*omega**2*self.phi**2*(1+eps*delta),simultenouos=True)

        mathieu_transformed_eq = mathieu_instability_eq.subs({self.l_w:1/eps,
                                                              self.phi.diff(self.ivar)*self.phi**2*self.y_e.diff(self.ivar):0,
                                                              self.z_e:A_z*cos(omega/p*self.ivar+Phi_z),
                                                              self.y_e:A_y*cos(omega/p*self.ivar+Phi_y),
                                                              A_y*self.phi**2*eps:0
                                                             })

        return mathieu_transformed_eq._eoms

#     def symbols_description(self):
#         self.sym_desc_dict = {
#             self.m_p: r'mass of payload \si{[\kilogram]}',
#             self.k_w: r'wire stiffness \si{[\newton\per\meter]}',
#             self.l_0: r'length of the lifting cable \si{[\metre]}',
#             tuple(self.q): r'generalized coordinates',
#             self.y_e:
#             r'lateral displacement at crane tip obtained from RAOs (a regular wave excitation) \si{[\metre]}',
#             self.z_e:
#             r'vertical displacement at crane tip obtained from RAOs (a regular wave excitation) \si{[\metre]}',
#             self.m_c: r'mass of compensator \si{[\kilogram]}',
#             self.k_c:
#             r'stiffness of heave compensator \si{[\newton\per\meter]}',
#             self.l_c:
#             r'length of the attached compensating element \si{[\metre]}',
#             self.g: r'acceleration of gravity \si{[\metre/\second\squared]}',
#             self.h_eq: r'equilibrium point of payload \si{[\metre]}',
#             self.h_ceq: r'equilibrium point of compensator \si{[\metre]}',
#             self.ivar: r'independent time variable',
#         }

#         return self.sym_desc_dict

    def symbols_description(self):

        parent_symbols_dict=super().symbols_description()

        self.sym_desc_dict = {** parent_symbols_dict , **{
            self.m_p: r'mass of payload,',
            self.k_w: r'wire stiffness,',
            self.l_0: r'length of the lifting cable,',
            self.l_w: r'length of the lifting cable-payload system,',
            tuple(self.q): r'payload generalized coordinates,',
            self.y_e:
            r'lateral displacement at crane tip obtained from RAOs (a regular wave excitation),',
            self.z_e:
            r'vertical displacement at crane tip obtained from RAOs (a regular wave excitation),',
            self.m_c: r'mass of compensator,',
            self.k_c:
            r'stiffness of heave compensator,',
            self.l_c:
            r'length of the attached compensating element,',
            self.g: r'acceleration of gravity,',
            self.h_eq: r'equilibrium point of payload,',
            self.h_ceq: r'equilibrium point of compensator,'
        }}

        return self.sym_desc_dict

    def numerical_data(self):
        numbers_dict={
            self.h_eq: ((self.m_c+self.m_p)/self.k_w+(self.m_p)/self.k_c)*self.g,
            self.h_ceq: self.g*(self.m_c+self.m_p)/self.k_w,
            self.m_p: 1e5,
            self.k_w: 4.09e6,
            self.l_0: self.l_w - self.l_c,
            self.y_e: 0.5 * cos(2/7 * pi * self.ivar -2.396),
            self.z_e: 1.5 * cos(2/7 * pi * self.ivar + 0.315),
            self.m_c: 1e4,
            self.k_c: 2.5e6,
            self.l_c:5,
            self.l_w:48,
            self.g: 9.81,
        }
        
        return numbers_dict    
    

    
    def units(self):
        units_dict={
            self.q[2]:ureg.meter,
            self.q[1]:ureg.meter,
            self.q[0]:ureg.radian,
            self.q[2].diff(self.ivar):ureg.meter/ureg.second,            
            self.q[1].diff(self.ivar):ureg.meter/ureg.second,
            self.q[0].diff(self.ivar):ureg.radian/ureg.second,

           }
        
        return units_dict 
# class PayloadVesselSystem(ComposedSystem):
    
#     def __init__(self,
#                  y_e=dynamicsymbols('y_e'),
#                  z_e=dynamicsymbols('z_e'),
#                  wave_level=dynamicsymbols('W'),
#                  wave_slope=dynamicsymbols('S'),
# #                  payload=TDoFCompensatedPayload(),
# #                  vessel=DDoFVessel(),
#                  system=None
#                 ):
        
#         self.payload = TDoFCompensatedPayload(y_e,z_e)
#         self.vessel = DDofVessel(wave_level,wave_slope)
        
#         system = self.payload + self.vessel
        
#         super().__init__(system)





class CompoundSystem(ComposedSystem):

    z = dynamicsymbols('z')
    _p = Symbol('p')


    @property
    def components(self):

        components = {}

        self._material_point = MaterialPoint(self._p, self.qs[0],
                                             self.qs)('Material Point')
        components['_material_point'] = self._material_point

        return components

    
class SDOFSemiSubmersiblePlatform(ComposedSystem):

    #scheme_name = '.PNG'
    #real_name = '.jpg'
    
    z=dynamicsymbols('z')
    m=Symbol('m', positive=True)
    m_p=Symbol('m_p', positive=True)
    k_1=Symbol('k', positive=True)
    c_1=Symbol('c', positive=True)
    c_2=Symbol('c_water', positive=True)
    F_load=Symbol('F_{load}', positive=True)
    g=Symbol('g', positive=True)
    p_p=Symbol('P_p', positive=True)
    rho=Symbol('\\rho', positive=True)
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    
    def __init__(self,
                 m=None,
                 m_p=None,
                 k_1=None,
                 c_1=None,
                 c_2=None,
                 F_load=None,
                 ivar=Symbol('t'),
                 z=None,
                 g=None,
                 p_p=None,
                 rho=None,
                 Omega=None,
                 F=None,
                 **kwargs):
        
        if z is not None: self.z=z
        if m is not None: self.m = m
        if k_1 is not None: self.k_1 = k_1
        if c_1 is not None: self.c_1 = c_1
        if c_2 is not None: self.c_2 = c_2
        if F_load is not None: self.F_load = F_load
        if g is not None: self.g=g
        if rho is not None: self.rho=rho
        if p_p is not None: self.p_p=p_p
        if Omega is not None: self.Omega=Omega
        if F is not None: self.F = F
        self.ivar=ivar
        
        self._init_from_components(**kwargs)
        
    @property
    def components(self):

        components = {}
        
        self._platform = MaterialPoint(self.m, pos1=self.z, qs=[self.z])(label='Material point - mass of the platform')
        self._spring_1 = Spring(self.k_1, pos1=self.z, pos2=(self.k_1*self.z)/(self.p_p*self.rho), qs=[self.z])(label='Spring - stiffness of the platform')
        self._spring_2 = Spring(self.p_p*self.rho, pos1=(self.k_1*self.z)/(self.p_p*self.rho), qs=[self.z])(label='Spring - buoyancy')
        self._damper = Damper(self.c_1, pos1=self.z, pos2=(self.k_1*self.z)/(self.p_p*self.rho), qs=[self.z])(label='Damper - damping of the platform')
        self._water = Damper(self.c_2, pos1=(self.k_1*self.z)/(self.p_p*self.rho), qs=[self.z])(label='Damper - damping of the water')
        self._force = Force(-self.F_load, pos1=self.z, qs=[self.z])(label='Force - load on the platform')
        self._MassOfPlatform = Force(-self.m*self.g, pos1=self.z, qs=[self.z])(label='Force - weight of the platform in gravitational field')
        self._excitation = Force(self.F*sin(self.Omega*self.ivar), pos1=(self.k_1*self.z)/(self.p_p*self.rho), qs=[self.z])(label='Force - Force caused by the waves')
        
        components['_platform'] = self._platform
        components['_spring_1'] = self._spring_1
        components['_spring_2'] = self._spring_2
        components['_damper'] = self._damper
        components['_water'] = self._water
        components['_force'] = self._force
        components['_MassOfPlatform'] = self._MassOfPlatform
        components['_excitation'] = self._excitation
        
        return components


    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Mass of a platform',
            self.g: r'Gravity field acceleration',
            self.Omega: r'Excitation frequency',
            self.c_1: r'Platform damping coefficient',
            self.c_2: r'Water damping coefficient',
            self.rho: r'Water density',
            self.k_1: r'Platform stiffness coefficient',
            self.p_p: r'Area submerged in water',
            self.F_load: r'Force caused by a load on a rig',
            self.F: r'Force caused by a waves',
            self.ivar: r'Time'
        }
        return self.sym_desc_dict
    
    
    def get_default_data(self):

        m0, Omega0, k0, p_p0, F0, lam0 = Symbol('m_0', positive=True), Symbol('Omega0', positive=True), Symbol('k0', positive=True), Symbol('p_{p0}', positive=True), Symbol('F0', positive=True), Symbol('lambda0', positive=True)
        c_2 = self.c_2
        g=self.g
        rho=self.rho

        default_data_dict = {
            self.m: [S.One * m0 * no * 1000 for no in range(20, 30)],
            self.g: [S.One * g * no for no in range(1, 2)],
            self.Omega: [S.One * Omega0 * no for no in range(1, 2)],
            self.c_2: [S.One * c_2 * no for no in range(1, 2)],
            self.rho: [S.One * rho * no for no in range(1, 2)],
            self.k_1: [S.One * k0 * no * 10000 for no in range(5, 10)],
            self.c_1: [S.One * k0 * lam0 * no * 10000 for no in range(5, 10)],
            self.p_p: [S.One * p_p0 * no for no in range(9, 16)],
            self.F_load: [S.One * F0 * no * 1000 for no in range(5, 10)],
            self.F: [S.One * F0 * no * 1000 for no in range(1, 10)]
        }

        return default_data_dict

    
    
class DDOFSemiSubmersiblePlatform(ComposedSystem):

    #scheme_name = '.PNG'
    #real_name = '.jpg'
    
    z_p=dynamicsymbols('z_p')
    z=dynamicsymbols('z')
    m=Symbol('m', positive=True)
    m_p=Symbol('m_p', positive=True)
    k_1=Symbol('k', positive=True)
    c_1=Symbol('c', positive=True)
    c_2=Symbol('c_p', positive=True)
    F_load=Symbol('F_{load}', positive=True)
    g=Symbol('g', positive=True)
    p_p=Symbol('P_p', positive=True)
    rho=Symbol('rho', positive=True)
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    
    def __init__(self,
                 m=None,
                 m_p=None,
                 k_1=None,
                 c_1=None,
                 c_2=None,
                 F_load=None,
                 ivar=Symbol('t'),
                 z=None,
                 z_p=None,
                 g=None,
                 p_p=None,
                 rho=None,
                 Omega=None,
                 F=None,
                 **kwargs):
        
        if z is not None: self.z=z
        if z_p is not None: self.z_p=z_p
        if m is not None: self.m = m
        if m_p is not None: self.m_p = m_p
        if k_1 is not None: self.k_1 = k_1
        if c_1 is not None: self.c_1 = c_1
        if c_2 is not None: self.c_2 = c_2
        if F_load is not None: self.F_load = F_load
        if g is not None: self.g=g
        if rho is not None: self.rho=rho
        if p_p is not None: self.p_p=p_p
        if Omega is not None: self.Omega=Omega
        if F is not None: self.F=F
        self.ivar=ivar
        
        self._init_from_components(**kwargs)
        
    @property
    def components(self):

        components = {}
        
        self._platform = MaterialPoint(self.m, pos1=self.z, qs=[self.z])(label='Material point - mass of the platform')
        self._pontoon = MaterialPoint(self.m_p, pos1=self.z_p, qs=[self.z_p])(label='Material point - mass of the pontoon')
        self._spring_1 = Spring(self.k_1, pos1=self.z, pos2=self.z_p, qs=[self.z, self.z_p])(label='Spring - stiffness of the platform')
        self._spring_2 = Spring(self.p_p*self.rho, pos1=self.z_p, qs=[self.z_p, self.z])(label='Spring - buoyancy')
        self._damper = Damper(self.c_1, pos1=self.z, pos2=self.z_p, qs=[self.z, self.z_p])(label='Damper - damping of the platform')
        self._water = Damper(self.c_2, pos1=self.z_p, qs=[self.z_p, self.z])(label='Damper - damping of the water')
        self._force = Force(-self.F_load, pos1=self.z, qs=[self.z, self.z_p])(label='Force - load on the platform')
        self._MassOfPlatform = Force(-self.m*self.g, pos1=self.z, qs=[self.z, self.z_p])(label='Force - weight of the platform in gravitational field')
        self._MassOfPontoon = Force(-self.m_p*self.g, pos1=self.z_p, qs=[self.z_p, self.z])(label='Force - weight of the pontoon in gravitational field')
        self._excitation = Force(self.F*sin(self.Omega*self.ivar), pos1=self.z_p, qs=[self.z_p, self.z])(label='Force - Force caused by the waves')
        
        
        components['_platform'] = self._platform
        components['_spring_1'] = self._spring_1
        components['_spring_2'] = self._spring_2
        components['_damper'] = self._damper
        components['_water'] = self._water
        components['_force'] = self._force
        components['_MassOfPlatform'] = self._MassOfPlatform
        components['_MassOfPontoon'] = self._MassOfPontoon
        components['_pontoon'] = self._pontoon
        components['_excitation'] = self._excitation
        
        return components
       
        
    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Mass of a platform',
            self.g: r'Gravity field acceleration',
            self.Omega: r'Excitation frequency',
            self.c_1: r'Platform damping coefficient',
            self.c_2: r'Water damping coefficient',
            self.rho: r'Water density',
            self.k_1: r'Platform stiffness coefficient',
            self.p_p: r'Area submerged in water',
            self.F_load: r'Force caused by a load on a rig',
            self.F: r'Force caused by a waves',
            self.ivar: r'Time',
            self.m_p: r'Mass of a pontoon'
        }
        return self.sym_desc_dict
    
    
    def get_default_data(self):

        m0, Omega0, k0, p_p0, F0, lam0 = Symbol('m_0', positive=True), Symbol('Omega0', positive=True), Symbol('k0', positive=True), Symbol('p_{p0}', positive=True), Symbol('F0', positive=True), Symbol('lambda_0', positive=True)
        c_2 = self.c_2
        g=self.g
        rho=self.rho

        default_data_dict = {
            self.m: [S.One * m0 * no * 1000 for no in range(20, 30)],
            self.m_p: [S.One * m0 * no * 1000 for no in range(10, 20)],
            self.g: [S.One * g * no for no in range(1, 2)],
            self.Omega: [S.One * Omega0 * no for no in range(1, 2)],
            self.c_2: [S.One * c_2 * no for no in range(1, 2)],
            self.rho: [S.One * rho * no for no in range(1, 2)],
            self.k_1: [S.One * k0 * no * 10000 for no in range(5, 10)],
            self.c_1: [S.One * k0 * lam0 * no * 10000 for no in range(5, 10)],
            self.p_p: [S.One * p_p0 * no for no in range(9, 16)],
            self.F_load: [S.One * F0 * 1000 * no  for no in range(5, 10)],
            self.F: [S.One * F0 * 1000 * no for no in range(1, 10)]
        }

        return default_data_dict
    
class ROVImpactDamped(ComposedSystem):

    scheme_name = 'engine.png'
    real_name = 'engine_real.PNG'

    m=Symbol('m', positive=True)
    m_n=Symbol('m_n', positive=True)
    c=Symbol('c',positive=True)
    k=Symbol('k', positive=True)
    ivar=Symbol('t')
    
    z, z_s=dynamicsymbols('z z_s')
    
    def __init__(self,
                 m=None,
                 c=None,
                 z=None,
                 z_s=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m is not None: self.m = m
        if c is not None: self.c = c
        if z is not None: self.z = z
        if z_s is not None: self.z_s = z_s

        self.ivar = ivar
        self.qs = [self.z,self.z_s]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self.material_point = MaterialPoint(self.m, self.z, qs=self.qs)
        self.damper = Damper(self.c, pos1=self.z_s, qs=self.qs)

        components['material_point'] = self.material_point
        components['damper'] = self.damper

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'mass of system on the spring',
            self.k: r'Spring coefficient ',
        }

        return self.sym_desc_dict

class ROVImpactElasticDamped(ROVImpactDamped):

    scheme_name = 'engine.png'
    real_name = 'engine_real.PNG'

    m=Symbol('m', positive=True)
    m_n=Symbol('m_n', positive=True)
    c=Symbol('c',positive=True)
    k=Symbol('k', positive=True)
    ivar=Symbol('t')
    
    z, z_s=dynamicsymbols('z z_s')
    
    def __init__(self,
                 m=None,
                 c=None,
                 k=None,
                 z=None,
                 z_s=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m is not None: self.m = m
        if k is not None: self.k = k
        if c is not None: self.c = c
        if z is not None: self.z = z
        if z_s is not None: self.z_s = z_s

        self.ivar = ivar
        self.qs = [self.z,self.z_s]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self.material_point = MaterialPoint(self.m, self.z, qs=self.qs)
        self.damper = Damper(self.c, pos1=self.z_s, qs=self.qs)
        self.spring = Spring(self.k, pos1=self.z, pos2=self.z_s, qs=self.qs)

        components['material_point'] = self.material_point
        components['damper'] = self.damper
        components['spring'] = self.spring

        return components
    
class ROVandStructureImpact(ROVImpactDamped):

    scheme_name = 'engine.png'
    real_name = 'engine_real.PNG'

    m=Symbol('m', positive=True)
    m_n=Symbol('m_n', positive=True)
    c=Symbol('c',positive=True)
    k=Symbol('k', positive=True)
    ivar=Symbol('t')
    
    z, z_s=dynamicsymbols('z z_s')
    
    def __init__(self,
                 m=None,
                 c=None,
                 k=None,
                 z=None,
                 z_s=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m is not None: self.m = m
        if k is not None: self.k = k
        if c is not None: self.c = c
        if z is not None: self.z = z
        if z_s is not None: self.z_s = z_s

        self.ivar = ivar
        self.qs = [self.z,self.z_s]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self.material_point = MaterialPoint(self.m, self.z, qs=self.qs)
        self.spring = Spring(self.k, pos1=self.z, pos2=self.z_s, qs=self.qs)
        self.damper = Damper(self.c, pos1=self.z_s, qs=self.qs)
        self.material_point_aux = MaterialPoint(self.m_n, self.z_s, qs=self.qs)

        components['material_point'] = self.material_point
        components['spring'] = self.spring
        components['damper'] = self.damper
        components['material_point1'] = self.material_point_aux

        return components

class ROVandStructureImpactMassModel(ROVandStructureImpact):

    scheme_name = 'engine.png'
    real_name = 'engine_real.PNG'

    m=Symbol('m', positive=True)
    m_n=Symbol('m_n', positive=True)
    k_n=Symbol('k_n', positive=True)
    c=Symbol('c', positive=True)
    k=Symbol('k', positive=True)
    ivar=Symbol('t')
    
    z, z_s=dynamicsymbols('z z_s')
    
    def __init__(self,
                 m=None,
                 m_n=None,
                 c=None,
                 k=None,
                 k_n=None,
                 z=None,
                 z_s=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m is not None: self.m = m
        if m_n is not None: self.m_n = m_n
        if k is not None: self.k = k
        if k_n is not None: self.k_n = k_n
        if c is not None: self.c = c
        if z is not None: self.z = z
        if z_s is not None: self.z_s = z_s

        self.ivar = ivar
        self.qs = [self.z,self.z_s]

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self.material_point = MaterialPoint(self.m, self.z, qs=self.qs)
        self.spring_rov = Spring(self.k, pos1=self.z, pos2=self.z_s, qs=self.qs)
        self.damper = Damper(self.c, pos1=self.z_s, qs=self.qs)
        self.material_point_aux = MaterialPoint(self.m_n, self.z_s, qs=self.qs)
        self.spring_str = Spring(self.k_n, pos1=self.z_s, pos2=0, qs=self.qs)

        components['material_point'] = self.material_point
        components['spring_rov'] = self.spring_rov
        components['damper'] = self.damper
        components['material_point1'] = self.material_point_aux
        components['spring_str'] = self.spring_str

        return components
    


from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin
from ...continuous import ContinuousSystem, PlaneStressProblem

import base64
import random
import IPython as IP
import numpy as np
import inspect

from functools import cached_property, lru_cache


from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST

from pint import UnitRegistry
ureg = UnitRegistry()



class RollingDisk(ComposedSystem):
    m1 = Symbol('m_1', positive=True)
    R = Symbol('R', positive=True)
    ivar = Symbol('t')
    x = dynamicsymbols('x')
    qs = dynamicsymbols('x')

    def __init__(self,
                 m1=None,
                 R=None,
                 x=None,
                 qs=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m1 is not None: self.m1 = m1
        if R is not None: self.R = R
        if x is not None: self.x = x

        if qs is None:
            self.qs = [self.x]
        else:
            self.qs = qs

        self.ivar=ivar

        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}

        self._disk_lin = MaterialPoint(self.m1, self.x, qs=self.qs)
        self._disk_rot = MaterialPoint(self.m1 / 2 * self.R**2, self.x / self.R, qs=self.qs)


        components['_disk_lin'] = self._disk_lin
        components['_disk_rot'] = self._disk_rot

        return components

#     def get_default_data(self):

#         m0, l0 = symbols('m_0 l_0', positive=True)

#         default_data_dict = {
#             self.m1: [S.One * m0 * no for no in range(1,20)],
#             self.R: [S.Half * l0 * no for no in range(1,20)],
#         }

#         return default_data_dict


class DiskMountingBlock(ComposedSystem):

    m = Symbol('m', positive=True)
    m1 = Symbol('m_1', positive=True)
    R = Symbol('R', positive=True)
    ivar = Symbol('t')
    x = dynamicsymbols('x')
    qs = dynamicsymbols('x')

    def __init__(self,
                 m=None,
                 m1=None,
                 R=None,
                 x=None,
                 qs=None,
                 ivar=Symbol('t'),
                 **kwargs):


        if m is not None: self.m = m
        if m1 is not None: self.m1 = m1
        if R is not None: self.R = R
        if x is not None: self.x = x
        if qs is None:
            self.qs = [self.x]
        else:
            self.qs = qs

        self.ivar=ivar

        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}

        self._disk = RollingDisk(m1=self.m, R=self.R, x=self.x, qs=[self.x], ivar=self.ivar)
        self._disk_inner = MaterialPoint(self.m1, self.x, qs = [self.x])


        components['_disk'] = self._disk
        components['_disk_inner'] = self._disk_inner

        return components

#     def get_default_data(self):

#         m0 = symbols('m_0', positive=True)

#         default_data_dict = {
#             self.m1: [S.One * m0 * no for no in range(1,20)],
#         }

#         return default_data_dict

#     def get_numerical_data(self):

#         m0 = symbols('m_0', positive=True)


#         default_data_dict = {
#             self.m1: [
#                 0.5 * m0, 1 * m0, 2 * m0, 3 * m0, 4 * m0, 5 * m0, 6 * m0,
#                 7 * m0, 8 * m0, 9 * m0
#             ],
#         }

#         return default_data_dict



class DiskWithNonlinearSpring(NonlinearComposedSystem):
    scheme_name = 'nonlinear_disc.png'
    real_name = 'roller_tightener.png'

    m1 = Symbol('m1', positive=True)
    kl = Symbol('k', positive=True)
    R = Symbol('R', positive=True)
    d = Symbol('d', positive=True)
    l_0 = Symbol('l_0', positive=True)
    ivar = Symbol('t')
    x = dynamicsymbols('x')

    def __init__(self,
                 m1=None,
                 kl=None,
                 R=None,
                 d=None,
                 l_0=None,
                 x=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m1 is not None: self.m1 = m1
        if kl is not None: self.kl = kl
        if R is not None: self.R = R
        if d is not None: self.d = d
        if l_0 is not None: self.l_0 = l_0
        if x is not None: self.x = x
        self.ivar=ivar
        self.qs = [self.x]
        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}

        self._disk = RollingDisk(self.m1, self.R, self.x, self.qs, self.ivar)
        self._spring = Spring(self.kl, pos1=(sqrt(self.x**2 + self.d**2) - self.l_0), qs=self.qs)


        components['_disk'] = self._disk
        components['_spring'] = self._spring
        return components


    def get_default_data(self):

        m0, k0, l0, F0 = symbols('m_0 k_0 l_0 F_0', positive=True)

        default_data_dict = {
            self.m1: [m0 * no for no in range(5, 15)],
            self.d: [self.l_0*S.Half*no for no in range(4,16)]
        }

        return default_data_dict



class ForcedDiskWithNonlinearSpring(NonlinearComposedSystem):
    scheme_name = 'nonlinear_disc.png'
    real_name = 'roller_tightener.png'

    m1 = Symbol('m1', positive=True)
    kl = Symbol('k', positive=True)
    R = Symbol('R', positive=True)
    d = Symbol('d', positive=True)
    l_0 = Symbol('l_0', positive=True)
    F = Symbol('F', positive=True)
    Omega = Symbol('Omega', positive=True)
    ivar = Symbol('t')
    x = dynamicsymbols('x')

    def __init__(self,
                 m1=None,
                 kl=None,
                 R=None,
                 d=None,
                 l_0=None,
                 F=None,
                 Omega=None,
                 x=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if m1 is not None: self.m1 = m1
        if kl is not None: self.kl = kl
        if R is not None: self.R = R
        if d is not None: self.d = d
        if l_0 is not None: self.l_0 = l_0
        if F is not None: self.F = F
        if Omega is not None: self.Omega = Omega
        if x is not None: self.x = x
        self.ivar=ivar
        self.qs = [self.x]
        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}

        self._disk = DiskWithNonlinearSpring(self.m1, self.kl, self.R, self.d, self.l_0, self.x, self.ivar)
        self._force = Force(self.F * cos(self.Omega * self.ivar), pos1=self.x, qs=self.qs)


        components['_disk'] = self._disk
        components['_force'] = self._force

        return components


class DampedDiskWithNonlinearSpring(NonlinearComposedSystem): #trzeba zupdateowac scheme - dodac tlumienie liniowe
    scheme_name = 'nonlinear_disc.png'
    real_name = 'roller_tightener.png'

    c = Symbol('c', positive=True)
    m1 = Symbol('m1', positive=True)
    kl = Symbol('k', positive=True)
    R = Symbol('R', positive=True)
    d = Symbol('d', positive=True)
    l_0 = Symbol('l_0', positive=True)
    F = Symbol('F', positive=True)
    Omega = Symbol('Omega', positive=True)
    ivar = Symbol('t')
    x = dynamicsymbols('x')
    qs = dynamicsymbols('x')

    def __init__(self,
                 m1=None,
                 c=None,
                 kl=None,
                 R=None,
                 d=None,
                 l_0=None,
                 F=None,
                 x=None,
                 qs=None,
                 ivar=Symbol('t'),
                 **kwargs):

        if c is not None: self.c = c
        if m1 is not None: self.m1 = m1
        if kl is not None: self.kl = kl
        if R is not None: self.R = R
        if d is not None: self.d = d
        if l_0 is not None: self.l_0 = l_0
        if F is not None: self.F = F
        if x is not None: self.x = x

        self._init_from_components(**kwargs)

    @cached_property
    def components(self):

        components = {}

        self._disk = ForcedDiskWithNonlinearSpring(self.m1, self.kl, self.R, self.d, self.l_0, self.F, self.Omega, self.x, self.ivar)
        self._damper = Damper(self.c, pos1 = self.x, pos2 = 0 , qs=[self.x])
        

        components['_disk'] = self._disk
        components['damper'] = self._damper
        
        return components



class ForcedNonLinearDisk(ForcedDiskWithNonlinearSpring):
    pass


class ForcedNonLinearDiskSpring(DampedDiskWithNonlinearSpring):
    pass



### Half Disk classes implementation


class RollingHalfDisk(ComposedSystem):
    """
    Model of a Single Degree of Freedom - RollingHalfDisc

        Arguments:
        =========
            m = Mass
                -Mass of the Half Disc

            g = gravitional field
                -value of gravitional field acceleration

            r = radius
                -radius of the Half Disc

            ivar = symbol object
                -Independant time variable

            phi = dynamicsymbol object
                -pendulation angle of the mass m

            qs = dynamicsymbol object
                -Generalized coordinates

        Example
        =======
        A half disc of radius R with optional horisontal force.

        >>> t = symbols('t')
        >>> R = symbols(' R ',positive=True)
        >>> RollingHalfDisc(r=R)

        -We define the symbols and dynamicsymbols
        -determine the instance of the RollingHalfDisc by using class RollingHalfDisc()
    """

    scheme_name = 'rolling_half_disk.png'
    real_name = 'rolling_half_disk.png'
    r=Symbol('r', positive=True)
    m=Symbol('m', positive=True)
    g=Symbol('g', positive=True)
    phi=dynamicsymbols('varphi')
    qs=[phi]

    def __init__(self,
                 r=None,
                 m=None,
                 g=None,
                 ivar=Symbol('t'),
                 phi=None,
                 qs=None,
                 **kwargs):


        if r is not None: self.r = r
        if m is not None: self.m = m
        if g is not None: self.g = g

        if phi is not None: self.phi = phi
        self.ivar=ivar
        
        if qs is not None: self.qs = qs
        
        self._init_from_components(**kwargs)

    @cached_property
    def components(self):
        components = {}

        d = 4*self.r/3/pi
        self.y = self.r - d*cos(self.phi)
        self.x =  self.r*self.phi - d*sin(self.phi)

        self.material_point_1 = MaterialPoint(self.m, self.x,ivar=self.ivar, qs=[self.phi])
        self.material_point_2 = MaterialPoint(self.m, self.y,ivar=self.ivar, qs=[self.phi])
        self.material_point_3 = MaterialPoint(((self.m*(self.r**2))/2)-self.m*d**2, self.phi,ivar=self.ivar, qs=[self.phi])
        self.gravity = GravitationalForce(self.m, self.g, pos1=self.y,ivar=self.ivar, qs=[self.phi])

        components['material_point_1'] = self.material_point_1
        components['material_point_2'] = self.material_point_2
        components['material_point_3'] = self.material_point_3
        components['gravity'] = self.gravity

        return components

    def get_default_data(self):

        m0, r0 = symbols('m_0 r_0', positive=True)

        default_data_dict = {
            self.m: [S.One * m0 * no for no in range(2,10)],
            self.r: [2 * r0, 3 * r0, 4 * r0, 5 * r0, 6 * r0],
        }
        return default_data_dict

    def get_numerical_data(self):
        m0, l0, r0 = symbols('m_0 l_0 r_0', positive=True)

        default_data_dict = {
            self.m: [1],
            self.r: [1],
        }
        return default_data_dict
    def symbols_description(self):
        self.sym_desc_dict = {
            self.r: r'Half disc radius',
            self.m: r'Mass',
            self.g: 'Gravity constant',
        }
        return self.sym_desc_dict
    
    def unit_dict(self):

        unit_dict = {

            self.r: ureg.meter,
            self.m: ureg.kilogram,
            self.g: ureg.meter/ureg.second/ureg.second,
            self.ivar: ureg.second,
            self.phi: ureg.radian,
            self.phi.diff(self.ivar): ureg.radian/ureg.second,
            self.phi.diff(self.ivar,2): ureg.radian/ureg.second/ureg.second,
        }
        return unit_dict

class ForcedRollingHalfDisk(RollingHalfDisk):
    """
    Model of a Forced Single Degree of Freedom - ForcedRollingHalfDisk

        Arguments:
        =========
            m = Mass
                -Mass of the Half Disc

            g = gravitional field
                -value of gravitional field acceleration

            r = radius
                -radius of the Half Disc
            
            M_0 = torque amplitude
                -amplitude value of torque
            
            Omega = Frequency of force
                -value of frequency force
            
            ivar = symbol object
                -Independant time variable

            phi = dynamicsymbol object
                -pendulation angle of the mass m

            qs = dynamicsymbol object
                -Generalized coordinates

        Example
        =======
        A half disc of radius R with optional horisontal force.

        >>> t = symbols('t')
        >>> R, Omega, M_0 = symbols(' R Omega M_0 ',positive=True)
        >>> ForcedRollingHalfDisk(r=R, Omega=Omega, M_0=M_0 )

        -We define the symbols and dynamicsymbols
        -determine the instance of the ForcedRollingHalfDisk by using class ForcedRollingHalfDisk()
    """

    scheme_name = 'forced_rolling_half_disk.png'
    real_name = 'forced_rolling_half_disk.png'
    r=Symbol('r', positive=True)
    l=Symbol('l', positive=True)
    m=Symbol('m', positive=True)
    g=Symbol('g', positive=True)
    Omega=Symbol('Omega', positive=True)
    M_0=Symbol('M_0', positive=True)
    phi=dynamicsymbols('varphi')

    def __init__(self,
                 r=None,
                 m=None,
                 g=None,
                 M_0=None,
                 Omega=None,
                 ivar=Symbol('t'),
                 phi=None,
                 **kwargs):

        if r is not None: self.r = r
        if m is not None: self.m = m
        if g is not None: self.g = g
        if phi is not None: self.phi = phi
        if M_0 is not None: self.M_0 = M_0
        if Omega is not None: self.Omega = Omega
        self.ivar=ivar
        self.qs=[self.phi]
        self._init_from_components(**kwargs)

    @cached_property
    def components(self):
        components = {}
        #self.rolling_half_disk.x
        d = 4*self.r/3/pi
        #self.y = self.r - d*cos(self.phi)
        x =  self.r*self.phi - d*sin(self.phi)
        self.rolling_half_disk = RollingHalfDisk(m=self.m, r=self.r, phi=self.phi, qs=self.qs)
        self.torque = Force(self.M_0*cos(self.Omega*self.ivar), self.phi, qs=self.qs)(label='force')

        components['rolling_half_disk'] = self.rolling_half_disk
        components['torque'] = self.torque
        
        return components

    def get_default_data(self):

        m0, r0 = symbols('m_0 r_0', positive=True)

        default_data_dict = {
            self.m: [S.One * m0 * no for no in range(2,10)],
            self.r: [2 * r0, 3 * r0, 4 * r0, 5 * r0, 6 * r0],
        }
        return default_data_dict

    def get_numerical_data(self):
        m0, l0, r0 = symbols('m_0 l_0 r_0', positive=True)

        default_data_dict = {
            self.m: [1],
            self.r: [1],
            self.M_0: [1],
            self.Omega: [1],
            #self.pi: [3.14],
        }
        return default_data_dict
    def symbols_description(self):
        self.sym_desc_dict = {
            self.r: r'Half disc radius',
            self.m: r'Mass',
            self.g: 'Gravity constant',
            self.M_0: 'Torque amplitude value',
            self.Omega: 'Frequency of torque'
        }
        return self.sym_desc_dict

    def unit_dict(self):

        unit_dict = {

            self.r: ureg.meter,
            self.m: ureg.kilogram,
            self.g: ureg.meter/ureg.second/ureg.second,
            self.ivar: ureg.second,
            self.phi: ureg.radian,
            self.phi.diff(self.ivar): ureg.radian/ureg.second,
            self.phi.diff(self.ivar,2): ureg.radian/ureg.second/ureg.second,
            self.M_0: ureg.newton*ureg.meter,
            self.Omega: ureg.radian/ureg.second,
        }
        return unit_dict
